/*
 * Datei:   create_timestamp
 * Autor:   Holger Klawitter
 * Datum:   12.11.1998
 * Projekt: Grab-Prfer/Convert
 */

#include <stdio.h>
#include <string.h>
#include <time.h>

typedef unsigned char byte;

typedef struct {
  char name[32];
  byte attr[2];
  byte version[2];
  byte cdate[4];
  byte mdate[4];
  byte backupdate[4];
  byte modno[4];
  byte appinfo[4];
  byte sortinfo[4];
  char dbType[4];
  char dbCreator[4];
  byte seed[4];
  byte nextRecList[4];
  char nRec[2];
} pdb_t;

typedef struct {
  byte data[4];
  char attr;
  byte id[3];
} recInfo_t;

void packInt( byte* dest, unsigned long l, int size ) {
	int i;
	for( i=size; i-->0; ) {
		dest[i] = l & 0x000000ff;
		l >>= 8;
	}
}

void packInt32( byte* dest, unsigned long l ) { packInt( dest, l, 4 ); }
void packInt16( byte* dest, unsigned long l ) { packInt( dest, l, 2 ); }

void fputdata( FILE* file, void* data, int size ) {
	int i;
	for( i=0; i<size; i++ ) {
		fputc( ((char*)data)[i], file );
	}
}

int main( int args, char* argv[] ) {
	pdb_t         pdb;
	unsigned long ctime;
	FILE*         file;
	unsigned long recPos;
  recInfo_t     recInfo;
	byte          data[4];

	file = fopen( "JustInTimeDB.pdb", "wb" );
	if( file == NULL ) { return 1; }

	ctime = (unsigned long)(time(NULL)) + 2082848400L - 3600 ;
	/* current time in secs since 1.1.1904 minus 1h (for debugging) */

	// write record header

	memset( &pdb, 0, sizeof(pdb_t) );
	strcpy( pdb.name, "JustInTimeDB" );
	memcpy( pdb.dbType, "Data", 4 );
	memcpy( pdb.dbCreator, "hk11", 4 );

	packInt32( pdb.cdate, ctime );
	packInt32( pdb.mdate, ctime );
	packInt16( pdb.nRec,  1 );

	fputdata( file, &pdb, sizeof(pdb_t) );

	// write record info

	memset( &recInfo, 0, sizeof(recInfo_t) );
	recPos = sizeof(pdb_t) + 1*sizeof(recInfo_t);
	packInt32( recInfo.data, recPos );
	packInt( recInfo.id, 1, 3 );
	fputdata( file, &recInfo, sizeof(recInfo_t) );

	// write record itself

	packInt32( data, ctime );
	fputdata( file, data, 4 );

	fclose( file );

	return 0;
}
