/*
 * File: justintime.c
 * Author: Holger Klawitter
 *
 * based on synctime from Judd Montgomery
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <pi-dlp.h>
#include <pi-file.h>
#include "libplugin.h"

int WriteTimeSyncInfo( FILE* file, int sd );

int plugin_get_name(char *name, int len)
{
   strncpy(name, "Just In Time 0.3", len);
   return 0;
}

int plugin_get_help_name(char *name, int len)
{
   strncpy(name, "About JustInTime", len);
   return 0;
}

int plugin_help(char **text, int *width, int *height)
{
   *text = strdup(
	   "(c) 2000 Holger Klawitter.\r\n"
	   "palm@klawitter.de\r\n"
	   "http://www.klawitter.de/palm/index.html\r\n"
	   );
   *height = 150;
   *width = 300;
}

#define TIMESYNCFILE "/tmp/JustInTimeDB.XXXXXX"

int plugin_sync(int sd)
{
	char*   tmpfilename;
	FILE*   tmpfile;
	mode_t  curmode;
   struct pi_file *f;

	jp_init();
   
	jp_logf(LOG_INFO, "justintime: installing timestamp on palm\n");

	//
	// Create a unique filename
	//
	tmpfilename = strdup( TIMESYNCFILE );
	if( mktemp( tmpfilename ) == NULL ) {
		jp_logf( LOG_WARN, "Couldn't create timetamp data file name.\n" );
		goto end;
	}

	//
	// Set up the file to correct and safe permissions, opens it.
	//
	curmode = umask( 0x077 );
	tmpfile = fopen( tmpfilename, "wb" );
	umask( curmode );
	if( tmpfile == NULL ) {
		jp_logf( LOG_WARN, "Couldn't create timestamp data file.\n" );
		goto end;
	}

	WriteTimeSyncInfo( tmpfile, sd );
	fclose( tmpfile );

	//
	// Push file to Palm Device
	//
	f = pi_file_open( tmpfilename );
	pi_file_install( f, sd, 0 );
	pi_file_close( f );

	unlink( tmpfilename );

 end:
	free( tmpfilename );
	return 0;
}

//////////////////////////////////////////////////////////////////////

typedef unsigned char byte;

typedef struct {
  byte data[4];
  char attr;
  byte id[3];
} recInfo_t;

typedef struct {
  char name[32];
  byte attr[2];
  byte version[2];
  byte cdate[4];
  byte mdate[4];
  byte backupdate[4];
  byte modno[4];
  byte appinfo[4];
  byte sortinfo[4];
  char dbType[4];
  char dbCreator[4];
  byte seed[4];
  byte nextRecList[4];
  char nRec[2];
} pdb_t;

void packInt( byte* dest, unsigned long l, int size ) {
	int i;
	for( i=size; i-->0; ) {
		dest[i] = l & 0x000000ff;
		l >>= 8;
	}
}

void packInt32( byte* dest, unsigned long l ) { packInt( dest, l, 4 ); }
void packInt16( byte* dest, unsigned long l ) { packInt( dest, l, 2 ); }

void fputdata( FILE* file, void* data, int size ) {
	int i;
	for( i=0; i<size; i++ ) {
		fputc( ((char*)data)[i], file );
	}
}

int WriteTimeSyncInfo( FILE* file, int sd ) {
	pdb_t         pdb;
	unsigned long pctime;
	unsigned long palmtime;
	unsigned long recPos;
    recInfo_t     recInfo;
	byte          data[4];

	pctime   = (unsigned long)(time(NULL));
	dlp_GetSysDateTime( sd, &palmtime );

	//
	// write record header
	//
	memset( &pdb, 0, sizeof(pdb_t) );
	strcpy( pdb.name, "JustInTimeDB" );
	memcpy( pdb.dbType, "Data", 4 );
	memcpy( pdb.dbCreator, "hk11", 4 );
	packInt16( pdb.nRec,  1 );
	fputdata( file, &pdb, sizeof(pdb_t) );

	//
	// write record info for one record
	//
	memset( &recInfo, 0, sizeof(recInfo_t) );
	recPos = sizeof(pdb_t) + 1*sizeof(recInfo_t); // + 2; // see below
	packInt32( recInfo.data, recPos );
	packInt( recInfo.id, 1, 3 );
	fputdata( file, &recInfo, sizeof(recInfo_t) );

	// normally, one would expect two zero bytes here,
	// but it seems, that they are not neccessary here.
	// If they are neccessary, still, you may add this:
	//   fputc( file, 0 ); fputc( file, 0 );
	// but remember to add two to the pointer in the record info block.

	//
	// write record itself
	//
	packInt32( data, pctime-palmtime );
	fputdata( file, data, 4 );

	return 0;
}

