unit DatabaseInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DocFile;

type
  TfrmDatabaseInfo = class(TForm)
    pbOK: TButton;
    dbRecInfo: TGroupBox;
    sgRecInfo: TStringGrid;
    gbDBFlags: TGroupBox;
    cbReadOnly: TCheckBox;
    cbDirty: TCheckBox;
    cbBackup: TCheckBox;
    cbOverwrite: TCheckBox;
    cbForceReset: TCheckBox;
    cbNoBeam: TCheckBox;
    gbDBInfo: TGroupBox;
    sgMoreInfo: TStringGrid;
    lbDBFileSize: TLabel;
    lbDBName: TLabel;
    txDBFileSize: TStaticText;
    cbCompressed: TCheckBox;
    edDBName: TEdit;
    pbCancel: TButton;
    procedure pbOKClick(Sender: TObject);
    procedure Refresh(Doc: TDocFile; EditMode: Boolean);
    procedure pbCancelClick(Sender: TObject);
  private
    Doc: TDocFile;
    EditMode: Boolean;
    procedure SizeGrids;
  end;

var
  frmDatabaseInfo: TfrmDatabaseInfo;


implementation

uses PalmDatabase;

{$R *.DFM}


//This gets an actual Doc reference; we can change the header information
//in edit mode
procedure TfrmDatabaseInfo.Refresh(Doc: TDocFile; EditMode: Boolean);
var
   	I: Integer;
begin
    Self.Doc := Doc;
    Self.EditMode := EditMode;

    //Flag fields
    edDBName.Text := Doc.DBHeader.Name;
    txDBFileSize.Caption := IntToStr(Doc.DBFileSize) + ' bytes';
    cbReadOnly.Checked := Doc.DBHeader.ReadOnly;
    cbDirty.Checked := Doc.DBHeader.Dirty;
    cbBackup.Checked := Doc.DBHeader.Backup;
    cbOverwrite.Checked := Doc.DBHeader.Overwrite;
    cbForceReset.Checked := Doc.DBHeader.ForceReset;
    cbNoBeam.Checked := Doc.DBHeader.NoBeam;
    cbCompressed.Checked := Doc.Compressed;

    //Info string grid
    with sgMoreInfo do
    begin
        //Set the headings
        Cells[0,0] := 'Records';
        Cells[0,1] := 'Type';
        Cells[0,2] := 'Creator';
        Cells[0,3] := 'Version';
        Cells[0,4] := 'Created';
        Cells[0,5] := 'Modified';
        Cells[0,6] := 'Backed up';
        Cells[0,7] := 'Mod number';
        Cells[0,8] := 'AppInfoArea';
        Cells[0,9] := 'SortInfoArea';
        Cells[0,10] := 'Next record list';
        Cells[0,11] := 'ID Seed';
        //Set the values
        Cells[1,0] := IntToStr(Doc.DBHeader.NumRecords);
        Cells[1,1] := Doc.DBHeader.DBType;
        Cells[1,2] := Doc.DBHeader.CreatorID;
        Cells[1,3] := IntToStr(Doc.DBHeader.Version);
        Cells[1,4] := DateToStr(Doc.DBHeader.Created);
        Cells[1,5] := DateToStr(Doc.DBHeader.Modified);
        Cells[1,6] := DateToStr(Doc.DBHeader.BackedUp);
        Cells[1,7] := IntToStr(Doc.DBHeader.Modification);
        Cells[1,8] := IntToStr(Doc.DBHeader.AppInfoArea);
        Cells[1,9] := IntToStr(Doc.DBHeader.SortInfoArea);
        Cells[1,10] := IntToStr(Doc.DBHeader.NextRecordList);
        Cells[1,11] := IntToStr(Doc.DBHeader.IDSeed);
    end;

    //Record info string grid
    with sgRecInfo do
    begin
        RowCount := (Doc.PDBRecordList.Count + 1);
        Cells[0,0] := 'Record';
        Cells[1,0] := 'Offset';
        Cells[2,0] := 'Size';
        for I := 0 to (Doc.PDBRecordList.Count - 1) do
        begin
            Cells[0, I+1] := IntToStr(I + 1);
            Cells[1, I+1] := IntToStr(TPDBRecordHeader(Doc.PDBRecordList.Items[I]).Offset);
            Cells[2, I+1] := IntToStr(TPDBRecordHeader(Doc.PDBRecordList.Items[I]).Size);
        end;
    end;

    SizeGrids;

    //Enable or disable changes
    edDBName.Enabled := EditMode;
    cbCompressed.Enabled := EditMode;
    pbCancel.Enabled := EditMode;
    gbDBFlags.Enabled := EditMode;
    for I:= 0 to gbDBFlags.ControlCount -1 do
        gbDBFlags.Controls[I].Enabled := EditMode;
end;


//Don't save changes made to header
procedure TfrmDatabaseInfo.pbCancelClick(Sender: TObject);
begin
    Close;
end;


//Save header changes (if editing is enabled)
procedure TfrmDatabaseInfo.pbOKClick(Sender: TObject);
begin
    if EditMode then
    begin
        Doc.DBHeader.Name := edDBName.Text;
        Doc.DBHeader.ReadOnly := cbReadOnly.Checked;
        Doc.DBHeader.Backup := cbBackup.Checked;
        Doc.DBHeader.OverWrite := cbOverwrite.Checked;
        Doc.DBHeader.ForceReset := cbForceReset.Checked;
        Doc.DBHeader.NoBeam := cbNoBeam.Checked;
        Doc.Compressed := cbCompressed.Checked;
    end;

    Close;
end;


procedure TfrmDatabaseInfo.SizeGrids;
var TextMetrics: TTextMetric;
    DC: HDC;
    OldFont: HFont;
    TextSize: TSize;
begin
    //First do the MoreInfo (top) grid
    DC := GetDC(sgMoreInfo.Handle);
    //Select the font into the DC and get the info
    OldFont := SelectObject(DC, Font.Handle);

    //2 different ways of doing this
    if not(GetTextMetrics(DC, TextMetrics)) then
        Exit;

    //Calculates width of the number '4', seems more accurate
    if not(GetTextExtentPoint32(DC, 'Next record list  ', 18, TextSize)) then
        Exit;

    //Now restore and release the DC
    SelectObject(DC, OldFont);
    ReleaseDC(sgMoreInfo.Handle, DC);
    sgMoreInfo.ColWidths[0] := TextSize.cx;
    sgMoreInfo.ColWidths[1] := sgMoreInfo.ClientWidth - sgMoreInfo.ColWidths[0] - sgMoreInfo.GridLineWidth;

    //Now do the RecInfo (bottom) grid
    DC := GetDC(sgRecInfo.Handle);
    //Select the font into the DC and get the info
    OldFont := SelectObject(DC, Font.Handle);

    //2 different ways of doing this
    if not(GetTextMetrics(DC, TextMetrics)) then
        Exit;

    //Calculates width of the number '4', seems more accurate
    if not(GetTextExtentPoint32(DC, 'Record  ', 8, TextSize)) then
        Exit;

    //Now restore and release the DC
    SelectObject(DC, OldFont);
    ReleaseDC(sgRecInfo.Handle, DC);
    sgRecInfo.ColWidths[0] := TextSize.cx;
    sgRecInfo.ColWidths[1] := Round((sgRecInfo.ClientWidth - sgRecInfo.ColWidths[0] - sgRecInfo.GridLineWidth) / 2);
    sgRecInfo.ColWidths[2] := sgRecInfo.ClientWidth - (sgRecInfo.ColWidths[0] + sgRecInfo.ColWidths[1] + (sgRecInfo.GridLineWidth * 2));

end;

end.
