unit DocReader;

//TODO:
{
	DocFile: Put in some error-handling stuff for file access

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus, inifiles;

type
	MRUEntry = record
		FileName: String;
        Position: Integer;
        MenuItem: Pointer;
    end;

  TfrmDocReader = class(TForm)
    memText: TMemo;
    mnuMainMenu: TMainMenu;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Search1: TMenuItem;
    Options1: TMenuItem;
    Help1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Copy2: TMenuItem;
    Find1: TMenuItem;
    Findnext1: TMenuItem;
    Font1: TMenuItem;
    About1: TMenuItem;
    dlgFont: TFontDialog;
    dlgOpen: TOpenDialog;
    Settings1: TMenuItem;
    Properties1: TMenuItem;
    Close1: TMenuItem;
    Top1: TMenuItem;
    Bottom1: TMenuItem;
    Goto1: TMenuItem;
    Go1: TMenuItem;
    Bookmark1: TMenuItem;
    MRUSeparator: TMenuItem;
    sbStatus: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Properties1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Settings1Click(Sender: TObject);
	procedure MRUOpenClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Exit1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Bottom1Click(Sender: TObject);
    procedure Top1Click(Sender: TObject);
  private
	procedure CreateMRUList;
	procedure UpdateMRUList;
   	procedure FreeMRUList;
	procedure OpenAndDisplayDoc(FileName: String);
	procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    { Private declarations }
  public
    DocDirectory: String;
  end;

var
	frmDocReader: TfrmDocReader;
   	FontName: String;
    FontSize: Integer;

implementation

uses DatabaseInfo, DocFile, Settings, shellapi;

{$R *.DFM}
var
    Doc: TDocFile;
    IniFile: TIniFile;
    MRUList: array[1..4] of MRUEntry;

procedure TfrmDocReader.FormCreate(Sender: TObject);
var
    I: Integer;

begin
	//Initialize globals
  	Self.memText.Text := 'No document loaded';

    //Read INI file
    //Create the TInifile object
    IniFile := TIniFile.Create(ExtractFileDir(Application.ExeName) + '.\docreader.ini');

    //Get the default directory
	DocDirectory := IniFile.ReadString('General', 'DefaultDirectory', '.\');

    //Read the MRU list
    for I := low(MRUList) to high(MRUList) do
    begin
    	MRUList[I].FileName := IniFile.ReadString('LastOpenFiles', Format('File%2.2d', [I]), '');
       	MRUList[I].Position := IniFile.ReadInteger('LastOpenPositions', Format('File%2.2d', [I]), 0);
    end;

    //Read the font information
    FontName := IniFile.ReadString('Font', 'Name', memText.Font.Name);
	FontSize := IniFile.ReadInteger('Font', 'Size', memText.Font.Size);

    //Set the font
    memText.Font.Name := FontName;
    memText.Font.Size := FontSize;

	//Create the menu entries for the MRU list
    CreateMRUList;
    UpdateMRUList;
end;


procedure TfrmDocReader.Open1Click(Sender: TObject);
begin
	//Get the open dialog ready
    dlgOpen.InitialDir := DocDirectory;
    dlgOpen.Filter := 'Doc files|*.PDB;*.PRC';

	//Present the open dialog
	if dlgOpen.Execute then
	begin
    	OpenAndDisplayDoc(dlgOpen.FileName);
		//If they changed directories, store the new directory for next time
		DocDirectory := ExtractFileDir(dlgOpen.FileName);
    end;
end;


procedure TfrmDocReader.Properties1Click(Sender: TObject);
begin
	frmDatabaseInfo.Refresh(Doc);
	frmDatabaseInfo.Show;
end;


procedure TfrmDocReader.Font1Click(Sender: TObject);
begin
  	dlgFont.Font := memText.Font; 
	if dlgFont.Execute then
    begin
    	memText.Font := dlgFont.Font;
       	FontName := dlgFont.Font.Name;
       	FontSize := dlgFont.Font.Size;
    end;
end;


procedure TfrmDocReader.Close1Click(Sender: TObject);
begin
	//Get rid of the doc object
	Doc.Free;
	Doc := nil;

	//Update the menu items
	Properties1.Enabled := False;
    Close1.Enabled := False;
   	Self.memText.Text := 'No document loaded';

	//Change the title bar and status bar
    Caption := 'Doc Reader';
	sbStatus.Panels[0].Text := '';
end;


procedure TfrmDocReader.Settings1Click(Sender: TObject);
begin
    frmSettings.DocDirectory := DocDirectory;
    frmSettings.FontName := FontName;
    frmSettings.FontSize := FontSize;

	if (frmSettings.ShowModal = mrOK) then
    begin
        FontName := frmSettings.FontName;
        FontSize := frmSettings.FontSize;
        memText.Font.Name := FontName;
        memText.Font.Size := FontSize;
        DocDirectory := frmSettings.DocDirectory;
    end;
end;


procedure TfrmDocReader.CreateMRUList;
var MRUMenuItem: TMenuItem;
    I, AddPosition: Integer;
begin
	//First, find the MRUSeparator menu item's index
	AddPosition := File1.IndexOf(MRUSeparator);

    //Now add as many items as we have MRU entries in the array
    for I := low(MRUList) to high(MRUList) do
    begin
		//Allocate a menu item
   	    MRUMenuItem := TMenuItem.Create(File1);
		//Set the event handler
        MRUMenuItem.OnClick := MRUOpenClick;
        //Set the tag; this is a reference to the MRUList index
        MRUMenuItem.Tag := I;
       	//Store off the reference
		MRUList[I].MenuItem := MRUMenuItem;
        //Add the menu item to the file menu
        File1.Insert(AddPosition + I, MRUMenuItem);
    end;
end;


procedure TfrmDocReader.FreeMRUList;
var
    I: Integer;
begin
	//Loop through and free the menu items
    for I := low(MRUList) to high(MRUList) do
    begin
		if MRUList[I].MenuItem <> nil then
	    begin
            File1.Remove(TMenuItem(MRUList[I].MenuItem));
            TMenuItem(MRUList[I].MenuItem).Free;
        end;
    end;
end;


procedure TfrmDocReader.UpdateMRUList;
var
    I: Integer;
begin
    for I := low(MRUList) to high(MRUList) do
    begin
		//Do we have a valid entry?
        if MRUList[I].FileName <> '' then
        begin
			//Change the name
    	    TMenuItem(MRUList[I].MenuItem).Caption := ExtractFileName(MRUList[I].FileName);
	        //Make it visible
			TMenuItem(MRUList[I].MenuItem).Visible := True;
            MRUSeparator.Visible := True;
		end
		else
        	TMenuItem(MRUList[I].MenuItem).Visible := False;
    end;
end;


procedure TfrmDocReader.MRUOpenClick(Sender: TObject);
begin
	//Find out which one was clicked, and what the file was they want
	if FileExists(MRUList[TMenuItem(Sender).Tag].FileName) then
		OpenAndDisplayDoc(MRUList[TMenuItem(Sender).Tag].FileName)
end;


procedure TfrmDocReader.OpenAndDisplayDoc(FileName: String);
var
	AlreadyThere: Boolean;
    I: Integer;
begin
	//If they already have one open, free the object first
	if Doc <> nil then
    begin
    	Doc.Free;
        Doc := nil;
    end;

	//Create the doc object
    Doc := TDocFile.Create(FileName);

	//Enable the associated menu items
	Properties1.Enabled := True;
	Close1.Enabled := True;

    //Change the title bar and status bar
    Caption := 'Doc Reader - ' + Doc.DBHeader.Name;
	sbStatus.Panels[0].Text := FileName;

    //Update the MRU list
	//See if it is already on the list
    AlreadyThere := False;
	for I := low(MRUList) to high(MRUList) do
    begin
    	if MRUList[I].FileName = FileName then
           	AlreadyThere := True;
    end;

    //Move everything down and put the new entry on top
    if not AlreadyThere then
    begin
		for I := high(MRUList) downto (low(MRUList)+1) do
		begin
           	MRUList[I].FileName := MRUList[I - 1].FileName;
           	MRUList[I].Position := MRUList[I - 1].Position;
        end;
		MRUList[low(MRUList)].FileName := FileName;
   		MRUList[low(MRUList)].Position := 0;
    end;

    //Update the file menu
    UpdateMRUList;

	//Display the document
	Self.memText.Text := Doc.GetDocument;
end;


procedure TfrmDocReader.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
	I: Integer;
begin
	//Release the document object
	if Doc <> nil then
	begin
        Doc.Free;
		Doc := nil;
    end;

	//Update the INI file
    //MRU entries
    for I := low(MRUList) to high(MRUList) do
    try
		IniFile.WriteString('LastOpenFiles', Format('File%2.2d', [I]), MRUList[I].FileName);
		IniFile.WriteInteger('LastOpenPositions', Format('File%2.2d', [I]), MRUList[I].Position);
    except
    end;

    //Font name and size
    try
        IniFile.WriteString('Font', 'Name', FontName);
        IniFile.WriteInteger('Font', 'Size', FontSize);
    except
    end;

	//Release the TIniFile object
	IniFile.Free;

    //Remove and release the MRU menu item entries
    FreeMRUList;
end;

procedure TfrmDocReader.Exit1Click(Sender: TObject);
begin
	Close;
end;


procedure TfrmDocReader.FormShow(Sender: TObject);
begin
	DragAcceptFiles(Handle, True);
end;


procedure TfrmDocReader.WMDropFiles(var Msg: TWMDropFiles);
var
  CFileName: array[0..MAX_PATH] of Char;
begin
	try
  		if DragQueryFile(Msg.Drop, 0, CFileName, MAX_PATH) > 0 then
		begin
			OpenAndDisplayDoc(CFileName);
			Msg.Result := 0;
	    end;
	finally
    	DragFinish(Msg.Drop);
	end;
end;


procedure TfrmDocReader.Bottom1Click(Sender: TObject);
begin
    memText.SelStart := length(memText.Text);
end;


procedure TfrmDocReader.Top1Click(Sender: TObject);
begin
    memText.SelStart := 0;
end;



end.
