unit DocReaderUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus, AboutBoxUnit, settings;

type
  TfrmDocReader = class(TForm)
    mnuMainMenu: TMainMenu;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Search1: TMenuItem;
    Options1: TMenuItem;
    Help1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Copy1: TMenuItem;
    Find1: TMenuItem;
    Findnext1: TMenuItem;
    Font1: TMenuItem;
    About1: TMenuItem;
    dlgFont: TFontDialog;
    dlgOpen: TOpenDialog;
    Settings1: TMenuItem;
    Properties1: TMenuItem;
    Close1: TMenuItem;
    MRUSeparator: TMenuItem;
    sbStatus: TStatusBar;
    reText: TRichEdit;
    dlgFind: TFindDialog;
    SelectAll1: TMenuItem;
    New1: TMenuItem;
    SaveAs1: TMenuItem;
    Save1: TMenuItem;
    dlgSave: TSaveDialog;
    N2: TMenuItem;
    N3: TMenuItem;
    Cut1: TMenuItem;
    Paste1: TMenuItem;
    prProgress: TProgressBar;
    N4: TMenuItem;
    EditMode1: TMenuItem;
    pmPopup: TPopupMenu;
    Cut2: TMenuItem;
    Copy2: TMenuItem;
    Paste2: TMenuItem;
    N5: TMenuItem;
    Find2: TMenuItem;
    EditMode2: TMenuItem;
    N6: TMenuItem;
    Bookmark1: TMenuItem;
    Add1: TMenuItem;
    Delete1: TMenuItem;
    Update1: TMenuItem;
    BookmarkSeparator: TMenuItem;
    MoreBookmarks1: TMenuItem;
    N7: TMenuItem;
    Bookmark2: TMenuItem;
    Rename1: TMenuItem;
    Print1: TMenuItem;
    dlgPrint: TPrintDialog;
    N8: TMenuItem;
    Install1: TMenuItem;
    Replace1: TMenuItem;
    dlgReplace: TReplaceDialog;
    procedure FormCreate(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Settings1Click(Sender: TObject);
    procedure MRUOpenClick(Sender: TObject);
    procedure BookmarkOpenClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Exit1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Find1Click(Sender: TObject);
    procedure dlgFindFind(Sender: TObject);
    procedure Findnext1Click(Sender: TObject);
    procedure reTextKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SelectAll1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure sbStatusResize(Sender: TObject);
    procedure sbStatusMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Properties1Click(Sender: TObject);
    procedure Cut1Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);
    procedure EditMode1Click(Sender: TObject);
    procedure pmPopupPopup(Sender: TObject);
    procedure Cut2Click(Sender: TObject);
    procedure Copy2Click(Sender: TObject);
    procedure Paste2Click(Sender: TObject);
    procedure Find2Click(Sender: TObject);
    procedure EditMode2Click(Sender: TObject);
    procedure Add1Click(Sender: TObject);
    procedure Update1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure MoreBookmarks1Click(Sender: TObject);
    procedure MoreBookmarks2Click(Sender: TObject);
    procedure Rename1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Install1Click(Sender: TObject);
    procedure File1Click(Sender: TObject);
    procedure Replace1Click(Sender: TObject);
    procedure dlgReplaceReplace(Sender: TObject);
    procedure dlgReplaceFind(Sender: TObject);
  private
    CurrentMRU: Integer;
    procedure CreateMRUMenu;
    procedure UpdateMRUMenu;
    procedure FreeMRUMenu;
    procedure AddToMRUList(FileName: String);
    procedure CreateBookmarkMenu;
    procedure FreeBookmarkMenu;
    procedure RebuildBookmarkMenu;
    procedure RebuildBookmarkPopup;
    procedure ScanForBookmarks;
    procedure OpenAndDisplayDoc(FileName: String; Position: Integer);
    procedure SetEditMode(NewEditMode: Boolean);
    procedure RefreshControls;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure W2KGlitch(Pos: Integer);
  end;

var
    frmDocReader: TfrmDocReader;

implementation

uses DatabaseInfo, DocFile, frmSettingsUnit, shellapi, math, frmBookmarkUnit, clipbrd,
  frmUserUnit, printers;

{$R *.DFM}
const
    BOOKMARK_MENU_PREFIX = 'BookmarkMenuJump';
    NO_DOC_TEXT = 'No document loaded' + #$D + #$A;
    TITLE_TEXT = 'Doc Reader';

var
    Doc: TDocFile;
    DocReaderSettings: TSettings;
    EditMode: Boolean = False;
    WarnedAboutPeanut: Boolean = False;


//////////////////////////////////////////////////////////////
//Form events
//
procedure TfrmDocReader.FormCreate(Sender: TObject);
var
    StartingDir, CurrentDir: String;
begin
    //Initialize globals
    reText.Clear;
    reText.Text := NO_DOC_TEXT;

    DocReaderSettings := TSettings.Create(ExtractFileDir(Application.ExeName) + '\docreader.ini');

    GetDir(0, CurrentDir);
    //Get the default directory
    //If the current directory is not the program directory (i.e., they
    //specified a shortcut with a different working directory), use that
    //instead of the default
    //Note: it will always be the directory of Palm Desktop on launch from the
    //Palm Desktop, so ignore that
    StartingDir := DocReaderSettings.DocDirectory;
    if (ParamCount <> 0) then
        if ParamStr(1) <> '-Desktop' then
            if CurrentDir <> ExtractFileDir(Application.ExeName) then
                StartingDir := CurrentDir;


    //Set up initial directories for dialogs
    dlgOpen.InitialDir := StartingDir;
    dlgSave.InitialDir := StartingDir;

    //Set the font and background color
    reText.Font.Assign(DocReaderSettings.Font);
    reText.DefAttributes.Assign(reText.Font);
    reText.Color := DocReaderSettings.Color;

    //Window size and position
    Top := DocReaderSettings.WindowTop;
    Left := DocReaderSettings.WindowLeft;
    Height := DocReaderSettings.WindowHeight;
    Width := DocReaderSettings.WindowWidth;

    //Editing mode
    SetEditMode(DocReaderSettings.EditMode);

    //Create the menu entries for the MRU list
    CreateMRUMenu;
    UpdateMRUMenu;

    //If they passed in a document on the command line, open it
    //(they can't pass a filename if they start from Palm Desktop, so
    //don't worry about it)
    if (ParamCount <> 0) then
        if FileExists(ParamStr(1)) then
            OpenAndDisplayDoc(ParamStr(1), 0);

    //Fix the RichEdit problem (can't paste more than 64K)
    reText.MaxLength := High(Integer) - 1024;
end;


procedure TfrmDocReader.FormShow(Sender: TObject);
begin
    DragAcceptFiles(Handle, True);
end;


procedure TfrmDocReader.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    //Release the document object
    if Doc <> nil then
        Close1Click(Sender);
    //They canceled out...
    if Doc <> nil then
        exit;

    FreeMRUMenu;

    //Save window size and position
    DocReaderSettings.WindowTop := Top;
    DocReaderSettings.WindowLeft := Left;
    DocReaderSettings.WindowHeight := Height;
    DocReaderSettings.WindowWidth := Width;

    //Update the INI file
    DocReaderSettings.WriteSettings;
    DocReaderSettings.Free;
end;


//////////////////////////////////////////////////////////////
//Object event handlers
//
procedure TfrmDocReader.sbStatusResize(Sender: TObject);
begin
    //Keep the panels in proportion
    sbStatus.Panels[0].Width := round(sbStatus.Width * 0.85);
end;


procedure TfrmDocReader.sbStatusMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    //If they clicked on the Read Only/Edit indicator, toggle the mode
    if (X > sbStatus.Panels[0].Width) and (Doc <> nil) then
        SetEditMode(not EditMode);
end;


procedure TfrmDocReader.reTextKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if Key = VK_F3 then
       dlgFindFind(Sender);
end;


procedure TfrmDocReader.dlgFindFind(Sender: TObject);
var
    FoundAt: LongInt;
    StartPos, ToEnd: Integer;
    SearchTypes: TSearchTypes;
begin
    //If they hit F3 and no search existed, run one
    if dlgFind.FindText = '' then
    begin
        dlgFind.Execute;
        exit;
    end;

    //Synchronize Find and Replace dialogs
    dlgReplace.FindText := dlgFind.FindText;
    dlgReplace.Options := dlgFind.Options;


    with reText do
    begin
        //Begin the search after the current selection if there is one
        //Otherwise, begin at the start of the text
        if SelLength <> 0 then
            StartPos := SelStart + SelLength
        else
            StartPos := SelStart;

        //ToEnd is the length from StartPos to the end of the text in the rich edit control
        ToEnd := Length(Text) - StartPos;

        //Use the options they selected in the Find dialog
        if frMatchCase in dlgFind.Options then
            Include(SearchTypes, stMatchCase);
        if frWholeWord in dlgFind.Options then
            Include(SearchTypes, stWholeWord);

        //Execute the search
        FoundAt := FindText(dlgFind.FindText, StartPos, ToEnd, SearchTypes);

        //If successful, highlight the text
        if FoundAt <> -1 then
        begin
            SetFocus;
            W2KGlitch(FoundAt);
            SelLength := Length(dlgFind.FindText);
            dlgFind.CloseDialog;
        end
        else
            ShowMessage('End of text reached');
    end;
end;


procedure TfrmDocReader.dlgReplaceFind(Sender: TObject);
begin
    //Set the Find dialog's values based on the Replace dialog and
    //call its Find method
    dlgFind.FindText := dlgReplace.FindText;
    dlgFind.Options := dlgReplace.Options;
    dlgFindFind(Sender);
end;


procedure TfrmDocReader.dlgReplaceReplace(Sender: TObject);
var
    FoundAt: LongInt;
    StartPos, ToEnd: integer;
    SearchTypes: TSearchTypes;
    ReplaceCount: Integer;
begin
    with reText do
    begin
        //Begin the search after the current selection if there is one
        //Otherwise, begin at the start of the text
        if SelLength <> 0 then
            StartPos := SelStart + SelLength
        else
            StartPos := SelStart;

        //ToEnd is the length from StartPos to the end of the text in the rich edit control
        ToEnd := Length(Text) - StartPos;

        //Use the options they selected in the Replace dialog
        if frMatchCase in dlgReplace.Options then
            Include(SearchTypes, stMatchCase);
        if frWholeWord in dlgReplace.Options then
            Include(SearchTypes, stWholeWord);

        //Do the search/replace
        ReplaceCount := 0;
        repeat
            FoundAt := FindText(dlgReplace.FindText, StartPos, ToEnd, SearchTypes);

            if FoundAt <> -1 then
            begin
                SetFocus;
                W2KGlitch(FoundAt);
                SelLength := Length(dlgReplace.FindText);
                //Replace selected text with ReplaceText
                SelText := dlgReplace.ReplaceText;
                Inc(ReplaceCount);
            end;
        until (FoundAt = -1) or not (frReplaceAll in dlgReplace.Options);

        //Show results
        if frReplaceAll in dlgReplace.Options then
            ShowMessage(IntToStr(ReplaceCount) + ' occurrences replaced.')
        else
            if ReplaceCount = 0 then
                ShowMessage('End of text reached');
    end;
end;


procedure TfrmDocReader.pmPopupPopup(Sender: TObject);
begin
    //Determine if anything is selected; if not, hide Cut and Copy options
    Cut2.Enabled := (reText.SelLength <> 0) and EditMode and (Doc <> nil);
    Copy2.Enabled := (reText.SelLength <> 0);
    Paste2.Enabled := EditMode and (Doc <> nil) and Clipboard.HasFormat(CF_TEXT);
end;


//////////////////////////////////////////////////////////////
//Menu handlers
//
//File menu
procedure TfrmDocReader.File1Click(Sender: TObject);
begin
    //Disable Install if document changed or if filename not .PDB or .PRC
    if Doc <> nil then
        Install1.Enabled := ((UpperCase(ExtractFileExt(Doc.PDBFile)) = '.PDB') or
       (UpperCase(ExtractFileExt(Doc.PDBFile)) = '.PRC')) and
       (not reText.Modified) and DocReaderSettings.DesktopInstalled
    else
        Install1.Enabled := False;
end;


procedure TfrmDocReader.New1Click(Sender: TObject);
var
    Title: String;
begin
    //Close any open document
    if Doc <> nil then
        Close1Click(Sender);
    //They canceled out...
    if Doc <> nil then
        exit;

    reText.Clear;

    //Create a new one
    if InputQuery('Enter Doc Title', 'Please enter the title for the new document', Title) then
        Doc := TDocFile.CreateNewDoc('', Title, True)
    else
        exit;

    SetEditMode(True);

    CurrentMRU := 0;
    Caption := TITLE_TEXT + ' - ' + Title;
    sbStatus.Panels[0].Text := 'New Document';
    FreeBookmarkMenu;
end;


procedure TfrmDocReader.Open1Click(Sender: TObject);
begin
    //Present the open dialog
    if dlgOpen.Execute then
    begin
        OpenAndDisplayDoc(dlgOpen.FileName, 0);
        dlgOpen.InitialDir := ExtractFileDir(dlgOpen.FileName);
    end;
end;


procedure UpdateProgressBar(Percent: Integer);
begin
    frmDocReader.prProgress.Position := Percent;
end;


procedure TfrmDocReader.Save1Click(Sender: TObject);
var
    ProgressCallback: TProgressCallback;
begin
    if Doc <> nil then
    begin
        if Doc.PDBFile = '' then
            if dlgSave.Execute then
            begin
                Doc.SetFile(dlgSave.FileName);
                dlgSave.InitialDir := ExtractFileDir(dlgSave.FileName);
            end
            else
                exit;

        //Write the document
        try
            sbStatus.Visible := False;
            prProgress.Visible := True;
            ProgressCallback := UpdateProgressBar;
            if UpperCase(ExtractFileExt(dlgSave.FileName)) = '.TXT' then
                reText.Lines.SaveToFile(dlgSave.FileName)
            else
                Doc.WriteDocument(reText.Text, ProgressCallback);
        finally
            prProgress.Visible := False;
            sbStatus.Visible := True;
        end;

        //Update other settings
        reText.Modified := False;
        AddToMRUList(Doc.PDBFile);
        sbStatus.Panels[0].Text := Doc.PDBFile;
    end;
end;


procedure TfrmDocReader.SaveAs1Click(Sender: TObject);
begin
    if Doc <> nil then
    begin
        if dlgSave.Execute then
        begin
            Doc.SetFile(dlgSave.FileName);
            Save1Click(Sender);
            dlgSave.InitialDir := ExtractFileDir(dlgSave.FileName);
        end;
    end;
end;


procedure TfrmDocReader.Close1Click(Sender: TObject);
begin
    //If they have changed the document, save it
    if reText.Modified then
        case MessageDlg('Do you want to save this Document?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
            mrYes:
                Save1Click(Sender);
            mrNo:
                reText.Modified := False;
            mrCancel:
                exit;
        end;

    //They canceled out...
    if reText.Modified then
        exit;

    //Get rid of the doc object
    Doc.Free;
    Doc := nil;

    //Update the menu items
    Properties1.Enabled := False;
    Close1.Enabled := False;

    //Update the MRU item with the position
    if CurrentMRU <> 0 then
        DocReaderSettings.MRUList[CurrentMRU].Position := reText.SelStart;

    //Change the title bar, status bar, and contents
    Self.reText.Text := NO_DOC_TEXT;
    Caption := TITLE_TEXT;
    sbStatus.Panels[0].Text := '';
    SetEditMode(EditMode);
    FreeBookmarkMenu;
end;


procedure TfrmDocReader.Properties1Click(Sender: TObject);
begin
    frmDatabaseInfo.Refresh(Doc, EditMode);
    frmDatabaseInfo.ShowModal;
    //In case they changed the name
    Caption := TITLE_TEXT + ' - ' + Doc.DBHeader.Name;
end;


procedure TfrmDocReader.MRUOpenClick(Sender: TObject);
begin
    //Find out which one was clicked, and what the file was they want
    if FileExists(DocReaderSettings.MRUList[TMenuItem(Sender).Tag].FileName) then
        OpenAndDisplayDoc(DocReaderSettings.MRUList[TMenuItem(Sender).Tag].FileName, DocReaderSettings.MRUList[TMenuItem(Sender).Tag].Position)
end;


procedure TfrmDocReader.Print1Click(Sender: TObject);
var
    PageWidth, PageHeight, DPIX, DPIY: Integer;
    RendRect: TRect;
begin
    if dlgPrint.Execute then
    try
        //Calculate margins
        PageWidth := GetDeviceCaps(Printer.Handle, PHYSICALWIDTH) - (GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX) * 2);
        PageHeight := GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT) - (GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY) * 2);
        DPIX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
        DPIY := GetDeviceCaps(Printer.Handle, LOGPIXELSY);
        RendRect.Top := Round(DocReaderSettings.TopMargin * DPIY) - GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);
        RendRect.Bottom := RendRect.Top + PageHeight - Round((DocReaderSettings.TopMargin + DocReaderSettings.BottomMargin) * DPIY) + (GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY) * 2);
        RendRect.Left := Round(DocReaderSettings.LeftMargin * DPIX) - GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);
        RendRect.Right := RendRect.Left + PageWidth - Round((DocReaderSettings.LeftMargin + DocReaderSettings.RightMargin) * DPIX) + (GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX) * 2);

        //Validate margins
        if (RendRect.Top < RendRect.Bottom) and (RendRect.Left < RendRect.Right) and
           (RendRect.Bottom <= PageHeight) and (RendRect.Right <= PageWidth) then
            reText.PageRect := RendRect;

        //Print
        reText.Print(Doc.DBHeader.Name);
    except
        MessageBox(Application.Handle, 'Error printing Document', 'Error', MB_OK or MB_ICONERROR);
    end;
end;


procedure TfrmDocReader.Install1Click(Sender: TObject);
begin
    frmUser.FileName := Doc.PDBFile;
    frmUser.ShowModal;
end;


procedure TfrmDocReader.Exit1Click(Sender: TObject);
begin
    Close;
end;


//Edit menu
procedure TfrmDocReader.Edit1Click(Sender: TObject);
begin
    Cut1.Enabled := EditMode and (reText.SelLength > 0) and (Doc <> nil);
    Copy1.Enabled := (reText.SelLength > 0);
    Paste1.Enabled := EditMode and (Doc <> nil) and Clipboard.HasFormat(CF_TEXT);
end;


procedure TfrmDocReader.Cut1Click(Sender: TObject);
begin
    reText.CutToClipboard;
end;


procedure TfrmDocReader.Copy1Click(Sender: TObject);
begin
    reText.CopyToClipboard;
end;


procedure TfrmDocReader.Paste1Click(Sender: TObject);
begin
    //reText.PasteFromClipboard works great, except it also pastes rich
    //formatting, which won't get rendered in the Doc.

    //Instead, this forces it to use text
    if Clipboard.HasFormat(CF_TEXT) then
        reText.SelText := Clipboard.AsText;
end;


procedure TfrmDocReader.SelectAll1Click(Sender: TObject);
begin
    reText.SelectAll;
end;


procedure TfrmDocReader.EditMode1Click(Sender: TObject);
begin
    SetEditMode(not EditMode);
end;


//Search menu
procedure TfrmDocReader.Find1Click(Sender: TObject);
begin
    dlgFind.Execute;
end;


procedure TfrmDocReader.Findnext1Click(Sender: TObject);
begin
    dlgFindFind(Sender);
end;


procedure TfrmDocReader.Replace1Click(Sender: TObject);
begin
    dlgReplace.Execute;
end;


//Options menu
procedure TfrmDocReader.Font1Click(Sender: TObject);
begin
    dlgFont.Font := reText.Font;
    if dlgFont.Execute then
        reText.Font := dlgFont.Font;
end;


procedure TfrmDocReader.Settings1Click(Sender: TObject);
begin
    frmSettings.Prepare(DocReaderSettings);
    if (frmSettings.ShowModal = mrOK) then
    begin
        reText.Font.Assign(DocReaderSettings.Font);
        reText.Color := DocReaderSettings.Color;
    end;
end;


//Bookmark menu
procedure TfrmDocReader.Add1Click(Sender: TObject);
var
    Name: String;
begin
    if InputQuery('Enter Bookmark Name', 'Please enter a name for the new bookmark', Name) then
    begin
        if (Length(Name) > MAX_BOOKMARK_NAME_LENGTH) then
        begin
            ShowMessage('Bookmark names can be a maximum of ' +
              IntToStr(MAX_BOOKMARK_NAME_LENGTH) + ' characters - truncating name.');
            Name := Copy(Name, 1, MAX_BOOKMARK_NAME_LENGTH);
        end;
        Doc.AddBookmark(Name, reText.SelStart);
        RebuildBookmarkMenu;
    end;
end;


procedure TfrmDocReader.Update1Click(Sender: TObject);
begin
    //Update a bookmark with the current position
    //Let them pick a bookmark
    frmBookmark.Caption := 'Update Bookmark';
    frmBookmark.FillListWithBookmarks(Doc.BookmarkList);
    if (frmBookmark.ShowModal = mrOK) then
        Doc.UpdateBookmark(frmBookmark.lbBookmarks.ItemIndex, reText.SelStart);
end;


procedure TfrmDocReader.Delete1Click(Sender: TObject);
var I: Integer;
begin
    //Delete the bookmark they select
    frmBookmark.Caption := 'Delete Bookmark';
    frmBookmark.FillListWithBookmarks(Doc.BookmarkList);
    //Let them pick multiple bookmarks
    frmBookmark.lbBookmarks.MultiSelect := True;
    if (frmBookmark.ShowModal = mrOK) then
    begin
        for I := (frmBookmark.lbBookmarks.Items.Count - 1) downto 0 do
            if frmBookmark.lbBookmarks.Selected[I] then
                Doc.DeleteBookmark(I);
        RebuildBookmarkMenu;
    end;
    frmBookmark.lbBookmarks.MultiSelect := False;
end;


procedure TfrmDocReader.Rename1Click(Sender: TObject);
var Name: String;
begin
    //Let them pick a bookmark
    frmBookmark.Caption := 'Rename Bookmark';
    frmBookmark.FillListWithBookmarks(Doc.BookmarkList);
    if (frmBookmark.ShowModal = mrOK) then
    begin
        //Get the new name
        if InputQuery('Enter Bookmark Name', 'Please enter the new name for this bookmark', Name) then
        begin
            if (Length(Name) > MAX_BOOKMARK_NAME_LENGTH) then
            begin
                ShowMessage('Bookmark names can be a maximum of ' +
                  IntToStr(MAX_BOOKMARK_NAME_LENGTH) + ' characters - truncating name.');
                Name := Copy(Name, 1, MAX_BOOKMARK_NAME_LENGTH);
            end;
            Doc.RenameBookmark(frmBookmark.lbBookmarks.ItemIndex, Name);
            RebuildBookmarkMenu;
        end;
    end;
end;


procedure TfrmDocReader.BookmarkOpenClick(Sender: TObject);
begin
    //Find out which one was clicked, and jump to that position
    W2KGlitch(TBookmark(Doc.BookmarkList.Items[TMenuItem(Sender).Tag]).Offset); 
end;


procedure TfrmDocReader.MoreBookmarks1Click(Sender: TObject);
begin
    //Let them pick a bookmark
    frmBookmark.Caption := 'Jump To Bookmark';
    frmBookmark.FillListWithBookmarks(Doc.BookmarkList);
    if (frmBookmark.ShowModal = mrOK) then
        W2KGlitch(TBookmark(Doc.BookmarkList.Items[frmBookmark.lbBookmarks.ItemIndex]).Offset);
end;


//Help menu
procedure TfrmDocReader.About1Click(Sender: TObject);
begin
    AboutBox.ShowModal;
end;


//Popup menu
procedure TfrmDocReader.Cut2Click(Sender: TObject);
begin
    Cut1Click(Sender);
end;


procedure TfrmDocReader.Copy2Click(Sender: TObject);
begin
    Copy1Click(Sender);
end;


procedure TfrmDocReader.Paste2Click(Sender: TObject);
begin
    Paste1Click(Sender);
end;


procedure TfrmDocReader.Find2Click(Sender: TObject);
begin
    Find1Click(Sender);
end;


procedure TfrmDocReader.EditMode2Click(Sender: TObject);
begin
    EditMode1Click(Sender);
end;


procedure TfrmDocReader.MoreBookmarks2Click(Sender: TObject);
begin
    MoreBookmarks1Click(Sender);
end;


//////////////////////////////////////////////////////////////
//Support functions
//
//Dynamically allocate the MRU menu items
procedure TfrmDocReader.CreateMRUMenu;
var MRUMenuItem: TMenuItem;
    I, AddPosition: Integer;
begin
    //First, find the MRUSeparator menu item's index
    AddPosition := File1.IndexOf(MRUSeparator);

    //Now add as many items as we have MRU entries in the array
    for I := low(DocReaderSettings.MRUList) to high(DocReaderSettings.MRUList) do
    begin
        //Allocate a menu item
        MRUMenuItem := TMenuItem.Create(File1);
        //Set the event handler
        MRUMenuItem.OnClick := MRUOpenClick;
        //Set the tag; this is a reference to the MRUList index
        MRUMenuItem.Tag := I;
        //Store off the reference
        DocReaderSettings.MRUList[I].MenuItem := MRUMenuItem;
        //Add the menu item to the file menu
        File1.Insert(AddPosition + I, MRUMenuItem);
    end;
end;


//Free the MRU menu items when finished
procedure TfrmDocReader.FreeMRUMenu;
var
    I: Integer;
begin
    //Loop through and free the menu items
    for I := low(DocReaderSettings.MRUList) to high(DocReaderSettings.MRUList) do
    begin
        if DocReaderSettings.MRUList[I].MenuItem <> nil then
        begin
            File1.Remove(TMenuItem(DocReaderSettings.MRUList[I].MenuItem));
            TMenuItem(DocReaderSettings.MRUList[I].MenuItem).Free;
        end;
    end;
end;


//Refresh the MRU menu items
procedure TfrmDocReader.UpdateMRUMenu;
var
    I: Integer;
begin
    for I := low(DocReaderSettings.MRUList) to high(DocReaderSettings.MRUList) do
    begin
        //Do we have a valid entry?
        if DocReaderSettings.MRUList[I].FileName <> '' then
        begin
            //Change the name
            TMenuItem(DocReaderSettings.MRUList[I].MenuItem).Caption := ExtractFileName(DocReaderSettings.MRUList[I].FileName);
            //Make it visible
            TMenuItem(DocReaderSettings.MRUList[I].MenuItem).Visible := True;
            MRUSeparator.Visible := True;
        end
        else
            TMenuItem(DocReaderSettings.MRUList[I].MenuItem).Visible := False;
    end;
end;


//Go through the MRU list and see if the file exists
//If so, set it as the CurrentMRU
//If not, add it to the list
procedure TfrmDocReader.AddToMRUList(FileName: String);
var
    AlreadyThere: Boolean;
    I: Integer;
begin
    //See if the file is already on the list
    AlreadyThere := False;
    for I := low(DocReaderSettings.MRUList) to high(DocReaderSettings.MRUList) do
    begin
        if UpperCase(DocReaderSettings.MRUList[I].FileName) = UpperCase(FileName) then
        begin
            AlreadyThere := True;
            CurrentMRU := I;
            break;
        end;
    end;

    //Move everything down and put the new entry on top
    if not AlreadyThere then
    begin
        for I := high(DocReaderSettings.MRUList) downto (low(DocReaderSettings.MRUList)+1) do
        begin
            DocReaderSettings.MRUList[I].FileName := DocReaderSettings.MRUList[I - 1].FileName;
            DocReaderSettings.MRUList[I].Position := DocReaderSettings.MRUList[I - 1].Position;
        end;
        DocReaderSettings.MRUList[low(DocReaderSettings.MRUList)].FileName := FileName;
        DocReaderSettings.MRUList[low(DocReaderSettings.MRUList)].Position := 0;
        CurrentMRU := low(DocReaderSettings.MRUList);
    end;

    //Update the menu items
    UpdateMRUMenu;
end;


//Dynamically allocate the Bookmark menu items
//We'll then copy the menu contents to the popup menu
procedure TfrmDocReader.CreateBookmarkMenu;
var BookmarkMenuItem: TMenuItem;
    I, AddPosition: Integer;
const
    MAX_DISPLAYED_BOOKMARKS = 15;
begin
    if Doc.BookmarkList.Count > 0 then
    begin
        BookmarkSeparator.Visible := True;
        MoreBookmarks1.Visible := False;

        //First, find the BookmarkSeparator menu item's index
        AddPosition := Bookmark1.IndexOf(BookmarkSeparator) + 1;

        //Now add as many items as we have Bookmark entries (stop at MAX_DISPLAYED_BOOKMARKS)
        for I := 0 to Min((Doc.BookmarkList.Count - 1), MAX_DISPLAYED_BOOKMARKS) do
        begin
            //Allocate a menu item
            BookmarkMenuItem := TMenuItem.Create(Bookmark1);
            BookmarkMenuItem.Caption := TBookmark(Doc.BookmarkList.Items[I]).Name;
            //Set the event handler
            BookmarkMenuItem.OnClick := BookmarkOpenClick;
            //Set the tag; this is a reference to the MRUList index
            BookmarkMenuItem.Tag := I;
            BookmarkMenuItem.Name := Format(BOOKMARK_MENU_PREFIX + '%3.3d', [I]);
            //Add the menu item to the Bookmark menu
            Bookmark1.Insert(AddPosition + I, BookmarkMenuItem);
        end;
        if (MAX_DISPLAYED_BOOKMARKS < (Doc.BookmarkList.Count - 1)) then
            MoreBookmarks1.Visible := True
    end
    else
        BookmarkSeparator.Visible := False;

    //Copy everything to the popup menu
    RebuildBookmarkPopup;
end;


//Free the Bookmark menu items when finished
procedure TfrmDocReader.FreeBookmarkMenu;
var
    I: Integer;
begin
    //Loop through and free the menu items
    I := 0;
    while I < (Bookmark1.Count - 1) do
    begin
        if Copy(Bookmark1.Items[I].Name, 1, Length(BOOKMARK_MENU_PREFIX)) = BOOKMARK_MENU_PREFIX then
            Bookmark1.Items[I].Free
        else
            Inc(I);
    end;

    BookmarkSeparator.Visible := False;
    MoreBookmarks1.Visible := False;
    RebuildBookmarkPopup;
end;


procedure TfrmDocReader.RebuildBookmarkMenu;
begin
    FreeBookmarkMenu;
    CreateBookmarkMenu;
    RefreshControls;
end;


//Delete anything in the Bookmark popup menu, then recopy it from
//the dropdown (just the jump items, though)
procedure TFrmDocReader.RebuildBookmarkPopup;
var
    I: Integer;
    NewMenuItem: TMenuItem;
begin
    //Delete existing menu items
    while Bookmark2.Count > 0 do
        Bookmark2.Items[0].Free;

    //Copy menu items
    for I := Bookmark1.IndexOf(BookmarkSeparator) to (Bookmark1.Count - 1) do
    begin
        //Don't copy the separator or More Bookmarks... items unless there are
        //actual bookmarks
        if (Bookmark2.Count > 0) or
          (Copy(Bookmark1.Items[I].Name, 1, Length(BOOKMARK_MENU_PREFIX)) = BOOKMARK_MENU_PREFIX) then
        begin
            //Create a new item
            NewMenuItem := TMenuItem.Create(Bookmark2);
            NewMenuItem.Caption := Bookmark1.Items[I].Caption;
            NewMenuItem.Tag := Bookmark1.Items[I].Tag;
            NewMenuItem.Visible := Bookmark1.Items[I].Visible;
            NewMenuItem.Enabled := Bookmark1.Items[I].Enabled;
            NewMenuItem.OnClick := Bookmark1.Items[I].OnClick;

            //Assign it to the menu (the menu will track the pointer...
            //we don't need to keep the reference)
            Bookmark2.Add(NewMenuItem);
        end;
    end;
    Bookmark2.Enabled := (Bookmark2.Count > 0);
end;


//Scan for text Bookmarks
procedure TfrmDocReader.ScanForBookmarks;
var
    BookmarkIdentifier, Line: String;
    I, Index, DocPosition: Integer;
begin
    //DocPosition keeps a count of our offset in the text, since
    //the Pos function used with a line gives us only the offset relative
    //to that line.
    DocPosition := 0;
    BookmarkIdentifier := Trim(reText.Lines[reText.Lines.Count - 1]);
    if BookmarkIdentifier <> '' then
    if (BookmarkIdentifier[1] = '<') and
       (BookmarkIdentifier[Length(BookmarkIdentifier)] = '>') then
    begin
        //The Delphi Lines property refers to the lines as they are DISPLAYED, not
        //a logical line (i.e. everything before a CRLF), so if we want to refer to
        //the logical line, we have to turn off WordWrap temporarily.  (How about
        //documenting that, Inprise???)
        reText.WordWrap := False;
        BookmarkIdentifier := Copy(BookmarkIdentifier, 2, Length(BookmarkIdentifier) - 2);
        for I := 0 to (reText.Lines.Count - 2) do
        begin
            Index := pos(BookmarkIdentifier, reText.Lines[I]);
            if Index > 0 then
            begin
                //Delete won't let you pass in reText.Lines[I]; it says it's a constant parameter (??)
                Line := reText.Lines[I];
                Delete(Line, Index, Length(BookmarkIdentifier));
                reText.Lines[I] := Line;
                Doc.AddBookmark(Copy(reText.Lines[I], Index, MAX_BOOKMARK_NAME_LENGTH), Index + DocPosition - 1);
            end;
            Inc(DocPosition, Length(reText.Lines[I]) + 2);  //CRLF
        end;
        //Get rid of the bookmark identifier on the last line
        reText.Lines.Delete(reText.Lines.Count - 1);
        RebuildBookmarkMenu;
        reText.WordWrap := True;        
    end;
end;


procedure TfrmDocReader.OpenAndDisplayDoc(FileName: String; Position: Integer);
begin
    //If they already have one open close it
    if Doc <> nil then
        Close1Click(Application);
    //They canceled out
    if Doc <> nil then
        exit;

    //If a text file, open it
    if UpperCase(ExtractFileExt(FileName)) = '.TXT' then
    begin
        reText.Lines.LoadFromFile(FileName);
        Doc := TDocFile.CreateNewDoc('', ExtractFileName(FileName), True);
        EditMode := True;
    end
    else
        //Create the doc object
        try
            Doc := TDocFile.Create(FileName);
        except
            on E: Exception do
            begin
                ShowMessage(E.Message);
                Doc.Free;
                Doc := nil;
                exit;
            end;
        end;

    //Change the title bar and status bar
    Caption := TITLE_TEXT + ' - ' + Doc.DBHeader.Name;
    sbStatus.Panels[0].Text := FileName;

    //Put it on the MRU list
    AddToMRUList(FileName);

    //Set our position in the document
    if Position = 0 then
        Position := DocReaderSettings.MRUList[CurrentMRU].Position;

    //Update the Bookmark menu
    RebuildBookmarkMenu;

    //Let the document know whether to display Peanut codes
    Doc.ShowPeanutCodes := DocReaderSettings.ShowPeanutCodes;

    //Display the document
    if UpperCase(ExtractFileExt(FileName)) <> '.TXT' then
        reText.Text := Doc.GetDocument;
    W2KGlitch(Position);

    //Scan for bookmarks
    if DocReaderSettings.BookmarkScan then
        ScanForBookmarks;

    reText.Modified := False;

    //Enable the associated menu items
    SetEditMode(EditMode);

    //If it's a Peanut document, display the warning
    if Doc.PeanutFile and not WarnedAboutPeanut then
    begin
        MessageDlg('Note: All formatting is removed from Peanut documents.', mtWarning, [mbOK], 0);
        WarnedAboutPeanut := True;
    end;
end;


//Process drop messages from Windows
procedure TfrmDocReader.WMDropFiles(var Msg: TWMDropFiles);
var
    CFileName: array[0..MAX_PATH] of Char;
begin
    try
        if DragQueryFile(Msg.Drop, 0, CFileName, MAX_PATH) > 0 then
        begin
            OpenAndDisplayDoc(CFileName, 0);
            Msg.Result := 0;
        end;
    finally
        DragFinish(Msg.Drop);
    end;
end;

//Update the form's controls when they switch between read-only and edit modes
procedure TfrmDocReader.SetEditMode(NewEditMode: Boolean);
begin
    EditMode := NewEditMode;
    RefreshControls;
end;


//Selectively enable/disable controls based on current status
procedure TfrmDocReader.RefreshControls;
var
    DocOpen: Boolean;
    DocHasBookmarks: Boolean;
begin
    DocOpen := (Doc <> nil);
    if DocOpen then
        DocHasBookmarks := (Doc.BookmarkList.Count > 0)
    else
        DocHasBookmarks := False;

    //File menu
    Close1.Enabled := DocOpen;
    Save1.Enabled := EditMode and DocOpen;
    SaveAs1.Enabled := DocOpen;
    Print1.Enabled := DocOpen;
    Properties1.Enabled := DocOpen;
    Install1.Enabled := DocOpen and not(reText.Modified) and DocReaderSettings.DesktopInstalled;
    //Edit menu
    EditMode1.Enabled := DocOpen;
    EditMode1.Checked := EditMode and DocOpen;
    EditMode2.Enabled := EditMode1.Enabled;
    EditMode2.Checked := EditMode1.Checked;
    //Bookmark menu
    Add1.Enabled := DocOpen;
    Update1.Enabled := EditMode and DocHasBookmarks;
    Rename1.Enabled := EditMode and DocHasBookmarks;
    Delete1.Enabled := EditMode and DocHasBookmarks;
    //Other
    reText.ReadOnly := not(EditMode and DocOpen);
    if (EditMode and DocOpen) then
        sbStatus.Panels[1].Text := 'Edit'
    else
    if (DocOpen) then
        sbStatus.Panels[1].Text := 'Read'
    else
        sbStatus.Panels[1].Text := '';

    RebuildBookmarkPopup;
end;


//Workaround for silly Windows 2000 behavior
procedure TfrmDocReader.W2KGlitch(Pos: Integer);
begin
    reText.SelStart := Pos;
    SendMessage(reText.Handle, WM_KEYDOWN, VK_RIGHT, 0);
    reText.SelStart := Pos;
end;

end.
