// --------------------------------------------------------
// TUserData Class for Delphi
// A Delphi class to auto-install PalmPilot .PDB and
// .PRC files.
//
// Copyright  1997 by Art Dahm
// All rights reserved
//
// Web:     http://palm.dahm.com
// E-mail:  palm@dahm.com
// --------------------------------------------------------
// Based loosely on the C++ TUserData class
// created by Mark Pierce (MakeDocW)
// --------------------------------------------------------
// Note: Changed MessageBox functions to Exceptions
// Search for "MP CHANGED"
unit AutoInst;

interface

uses
	 Windows, SysUtils;

type
	PUsers = ^TUsers;

	TUsers = record
		Magic: Word;
		Name: Array[0..31] of Char;
		SubDir: Array[0..31] of Char;
	end;

	TUserData = class
	private
		fUsers: Byte;
		fLastUserNumber: Integer;
		fCore: Array[0..255] of Char;
		fLastUserName: Array[0..31] of Char;
		fUserList: Pointer;
		procedure	ReadUserData;
	public
		constructor	Create;
		destructor	Destroy; override;
		function	GetNumUsers: Integer;
		function	GetLastUserNumber: Integer;
		function	GetCore: String;
		function	GetUserMagic(User: Integer): Integer;
		function	GetUserName(User: Integer): String;
		function	GetUserSubDir(User: Integer): String;
		function	InstallUser(User: Integer; FileName: String): Boolean;
	end;

 //MP CHANGED added next line
 EPalmDesktopError = class(Exception);

const
	sizeofTUsers = sizeof(TUsers);

implementation

constructor TUserData.Create;
var
	hkeyhnd: HKEY;
	hkeyUSR: HKEY;
	dwLen: DWORD;
	dwType: DWORD;
begin
	inherited;
	fUserList := nil;
	fUsers := 0;
	fLastUserNumber := 0;
	fLastUserName := '';
	hkeyUSR := 0;

	// First look for DeskTop 2.0 Entries...
	if (RegOpenKey(HKEY_CURRENT_USER, 'Software\U.S. Robotics', hkeyUSR) <> ERROR_SUCCESS) then
	begin
		// Not found, look for DeskTop 1.0 entry....
		if (RegOpenKey(HKEY_CURRENT_USER, 'Software\Palm Computing', hkeyUSR) <> ERROR_SUCCESS) then
		begin
   //MP CHANGED Commented this line, added following raise
			//MessageBox(0, 'The PalmPilot Desktop software is not correctly installed.', 'Image Converter', MB_ICONSTOP or MB_OK);
			hkeyUSR := 0;
   raise EPalmDesktopError.Create('The PalmPilot Desktop software is not correctly installed.');
		end;
	end;

	if (hkeyUSR <> 0) then
	begin
		if (RegOpenKey(hkeyUSR, 'Pilot Desktop\Preferences', hkeyhnd) = ERROR_SUCCESS) then
		begin
			dwLen := sizeof(fLastUserName);
			dwType := REG_SZ;
			RegQueryValueEx(hkeyhnd, 'LastUserName', nil, @dwType, @fLastUserName, @dwLen);
			RegCloseKey(hkeyhnd);
		end;

		if (RegOpenKey(hkeyUSR, 'Pilot Desktop\Core', hkeyhnd) = ERROR_SUCCESS) then
		begin
			dwLen := sizeof(fCore);
			dwType := REG_SZ;
			if (RegQueryValueEx(hkeyhnd, 'Path', nil, @dwType, @fCore, @dwLen) = ERROR_SUCCESS) then
				ReadUserData();
			RegCloseKey(hkeyhnd);
		end;

		RegCloseKey(hkeyUSR);
	end;
end;

destructor	TUserData.Destroy;
begin
	if fUserList <> nil then
		FreeMem(fUserList);
	fUsers := 0;
	inherited;
end;

procedure	TUserData.ReadUserData;
var
	strUserData: String;
	fUserData: file;
	dwStrLen: Byte;
	lnOffset: LongInt;
	n: Integer;
	plistitem: PUsers;
	FoundUser: Boolean;
begin
	strUserData := fCore + '\users.dat';
	AssignFile(fUserData, strUserData);
	try
		lnOffset := 0;
		Reset(fUserData, 1);
		Seek(fUserData, lnOffset);
		BlockRead(fUserData, fUsers, 1);

		if (fUsers > 0) then
		begin
			GetMem(fUserList, sizeofTUsers*fUsers);

			if (fUserList <> nil) then
			begin
				// Offset of first user entry
				lnOffset := 13;
				plistitem := PUsers(fUserList);

				for n := 0 to fUsers-1 do
				begin
					dwStrLen := 0;
					plistitem^.Magic := 0;

					// Read the user's magic number
					Seek(fUserData, lnOffset);
					BlockRead(fUserData, plistitem^.Magic, 2);

					// Read the user's name
					Inc(lnOffset,4);
					Seek(fUserData, lnOffset);
					BlockRead(fUserData, dwStrLen, 1);
					BlockRead(fUserData, plistitem^.Name, dwStrLen);
					plistitem^.Name[dwStrLen] := Chr(0);
					lnOffset := lnOffset + (dwStrLen + 1);

					// Check if this was the last user
					if (StrComp(PChar(@(fLastUserName[0])), PChar(@(plistitem^.Name[0]))) = 0) then
						fLastUserNumber := n;

					// Read the user's sub-dir
					BlockRead(fUserData, dwStrLen, 1);
					BlockRead(fUserData, plistitem^.SubDir, dwStrLen);
					plistitem^.SubDir[dwStrLen] := chr(0);
					lnOffset := lnOffset + (dwStrLen + 1);

					// Read the flag byte
					BlockRead(fUserData, dwStrLen, 1);

					// Set offset of next user entry
					if n < (fUsers-1) then
					begin
						Inc(lnOffset, 1);
						FoundUser := False;
						while not FoundUser do
						begin
							Inc(lnOffset, 1);
							BlockRead(fUserData, dwStrLen, 1);
							if dwStrLen = $01 then
							begin
								Inc(lnOffset, 1);
								BlockRead(fUserData, dwStrLen, 1);
								if dwStrLen = $80 then
									FoundUser := True;
							end;
						end;
					end;
{					if dwStrLen <> 0 then
//						Inc(lnOffset, 42)
						Inc(lnOffset, 43)
					else
						Inc(lnOffset, 10);
					Inc(lnOffset,2);	// Skip the 0x01, 0x80 bytes at the start of the next record... }
					Inc(plistitem);
				end;
			end;
		end;
	finally
		CloseFile(fUserData);
	end;
end;

function	TUserData.GetNumUsers: Integer;
begin
	result := fUsers;
end;

function	TUserData.GetLastUserNumber: Integer;
begin
	result := fLastUserNumber;
end;

function	TUserData.GetCore: String;
begin
	result := fCore;
end;

function	TUserData.GetUserMagic(User: Integer): Integer;
var
	p: PUsers;
begin
	if ((fUserList <> nil) and (User >= 0) and (User < fUsers)) then
	begin
		p := fUserList;
		Inc(p, User);
		result := p^.Magic;
	end
	else
		result := 0;
end;

function	TUserData.GetUserName(User: Integer): String;
var
	p: PUsers;
begin
	if ((fUserList <> nil) and (User >= 0) and (User < fUsers)) then
	begin
		p := fUserList;
		Inc(p, User);
		result := p^.Name;
	end
	else
		result := '';
end;

function	TUserData.GetUserSubDir(User: Integer): String;
var
	p: PUsers;
begin
	if ((fUserList <> nil) and (User >= 0) and (User < fUsers)) then
	begin
		p := fUserList;
		Inc(p, User);
		result := p^.SubDir;
	end
	else
		result := '';
end;

function	TUserData.InstallUser(User: Integer; FileName: String): Boolean;
var
	bResult: Boolean;
	hkeyhnd: HKEY;
	hkeyUSR: HKEY;
	strTemp: String;
	strParam: String;
	dwValue: Integer;
begin
	bResult := False;
	if (GetNumUsers > 0) then
	begin
		hkeyUSR := 0;

		strTemp := GetCore() + '\' + GetUserSubDir(User) + '\Install';
		CreateDirectory(PChar(strTemp), nil);

		strTemp := strTemp + '\' + ExtractFileName(FileName);

		if CopyFile(PChar(FileName), PChar(strTemp), False) then
		begin
			// First look for DeskTop 2.0 Entries...
			if (RegOpenKey(HKEY_CURRENT_USER, 'Software\U.S. Robotics', hkeyUSR) <> ERROR_SUCCESS) then
			begin
				// Not found, look for DeskTop 1.0 entry....
				if (RegOpenKey(HKEY_CURRENT_USER, 'Software\Palm Computing', hkeyUSR) <> ERROR_SUCCESS) then
				begin
     //MP CHANGED Commented this line, added following Result := and raise
					//MessageBox(0, 'The Pilot DeskTop Software is not Correctly Installed', 'Image Converter', MB_ICONSTOP or MB_OK);
					hkeyUSR := 0;
     Result := bResult;
     raise EPalmDesktopError.Create('The PalmPilot Desktop software is not correctly installed.');
				end;
			end;

			if (hkeyUSR <> 0) then
			begin
					if RegOpenKey(hkeyUSR, 'Pilot Desktop\HotSync Manager', hkeyhnd) = ERROR_SUCCESS then
					begin
						dwValue := 1;

						strParam := 'Install' + IntToStr(GetUserMagic(User));
						RegSetValueEx(hkeyhnd, PChar(strParam), 0, REG_DWORD, @dwValue, sizeof(DWORD));

						RegCloseKey(hkeyhnd);
						bResult := True;
					end;
					RegCloseKey(hkeyUSR);
			end;
		end;
	end;
	Result := bResult;
end;

end.
