unit frmSettingsUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl, ExtCtrls, settings;

type
    TfrmSettings = class(TForm)
    lbDefaultDir: TLabel;
    Label1: TLabel;
    edDefaultDir: TEdit;
    pbSelectDir: TButton;
    pbSelectFont: TButton;
    stDefaultFont: TStaticText;
    pbOK: TButton;
    pbCancel: TButton;
    lbDefaultFont: TLabel;
    Label2: TLabel;
    pnColor: TPanel;
    pbSelectColor: TButton;
    dlgColor: TColorDialog;
    dlgFont: TFontDialog;
    cbEditMode: TCheckBox;
    cbBookmarkScan: TCheckBox;
    gbMargins: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    edTop: TEdit;
    edLeft: TEdit;
    Label5: TLabel;
    edBottom: TEdit;
    Label6: TLabel;
    edRight: TEdit;
    cbIntegrate: TCheckBox;
    procedure pbSelectDirClick(Sender: TObject);
    procedure pbSelectFontClick(Sender: TObject);
    procedure Prepare(Settings: TSettings);
    procedure FormShow(Sender: TObject);
    procedure pbSelectColorClick(Sender: TObject);
    procedure pnColorClick(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure edMarginsExit(Sender: TObject);
    procedure edDefaultDirExit(Sender: TObject);
  public
    DocDirectory: String;
  private
    Settings: TSettings;
    function GetValidString(MarginIn: String): String; overload;
    function GetValidString(MarginIn: Real): String; overload;
  end;

var
  frmSettings: TfrmSettings;

implementation

{$R *.DFM}

uses DocReaderUnit, ShellAPI, ShlObj, DesktopIntegrate;

//I don't think SelectDirectory was in Delphi 3; see below for the
//Windows API implementation
procedure TfrmSettings.pbSelectDirClick(Sender: TObject);
begin
    if SelectDirectory('Select default Doc directory', '', DocDirectory) then
    begin
        if DocDirectory[Length(DocDirectory)] <> '\' then
            DocDirectory := DocDirectory + '\';
        edDefaultDir.Text := DocDirectory;
    end;
end;


procedure TfrmSettings.pbSelectFontClick(Sender: TObject);
begin
    if dlgFont.Execute then
    begin
        lbDefaultFont.Caption := dlgFont.Font.Name + ' ' + IntToStr(dlgFont.Font.Size);
        lbDefaultFont.Font.Color := dlgFont.Font.Color;
    end;
end;


procedure TfrmSettings.pbSelectColorClick(Sender: TObject);
begin
    dlgColor.Color := pnColor.Color;
    if dlgColor.Execute then
        pnColor.Color := dlgColor.Color;
end;


//Save a reference to the settings that the main form gives us
//Note: This unit assumes the reference will not change through the life
//of the program.
procedure TfrmSettings.Prepare(Settings: TSettings);
begin
    Self.Settings := Settings;
end;


//Set the initial values of the controls based on the Settings class
procedure TfrmSettings.FormShow(Sender: TObject);
begin
    edDefaultDir.Text := Settings.DocDirectory;
    lbDefaultFont.Caption := Settings.Font.Name + ' ' + IntToStr(Settings.Font.Size);
    lbDefaultFont.Font.Color := Settings.Font.Color;
    dlgFont.Font.Assign(Settings.Font);
    pnColor.Color := Settings.Color;
    cbEditMode.Checked := Settings.EditMode;
    cbBookmarkScan.Checked := Settings.BookmarkScan;
    cbIntegrate.Enabled := Settings.DesktopInstalled;
    cbIntegrate.Checked := Settings.IntegrateWithDesktop;
    edTop.Text := GetValidString(Settings.TopMargin);
    edBottom.Text := GetValidString(Settings.BottomMargin);
    edLeft.Text := GetValidString(Settings.LeftMargin);
    edRight.Text := GetValidString(Settings.RightMargin);
end;


procedure TfrmSettings.pnColorClick(Sender: TObject);
begin
    pbSelectColorClick(Sender);
end;


procedure TfrmSettings.pbOKClick(Sender: TObject);
begin
    //Store the updated values in the Settings class
    with Settings do
    begin
        DocDirectory := edDefaultDir.Text;
        Font.Assign(dlgFont.Font);
        Color := pnColor.Color;
        EditMode := cbEditMode.Checked;
        BookmarkScan := cbBookmarkScan.Checked;
        TopMargin := StrToFloat(edTop.Text);
        BottomMargin := StrToFloat(edBottom.Text);
        LeftMargin := StrToFloat(edLeft.Text);
        RightMargin := StrToFloat(edRight.Text);
        if IntegrateWithDesktop <> cbIntegrate.Checked then
        begin
            IntegrateWithDesktop := cbIntegrate.Checked;
            if IntegrateWithDesktop then
                Integrate
            else
                Disintegrate;
        end;
    end;
end;


procedure TfrmSettings.edMarginsExit(Sender: TObject);
begin
    //Make sure valid data goes in
    TEdit(Sender).Text := GetValidString(TEdit(Sender).Text);
end;


function TfrmSettings.GetValidString(MarginIn: String): String;
begin
    //Make sure valid data goes in
    try
        Result := Trim(Format('%4.2f', [StrToFloat(MarginIn)]))
    except
        Result := '0'
    end;
end;

function TfrmSettings.GetValidString(MarginIn: Real): String;
begin
    Result := GetValidString(FloatToStr(MarginIn));
end;

procedure TfrmSettings.edDefaultDirExit(Sender: TObject);
begin
    if edDefaultDir.Text[length(edDefaultDir.Text)] <> '\' then
        edDefaultDir.Text := edDefaultDir.Text + '\';
end;

end.

{
procedure TfrmSettings.pbSelectDirClick(Sender: TObject);
var
    TitleName : string;
    lpItemID : PItemIDList;
    BrowseInfo : TBrowseInfo;
    DisplayName : array[0..MAX_PATH] of char;
    TempPath : array[0..MAX_PATH] of char;
begin
    FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
    BrowseInfo.hwndOwner := Self.Handle;
    BrowseInfo.pszDisplayName := @DisplayName;
    TitleName := 'Please specify a directory';
    BrowseInfo.lpszTitle := PChar(TitleName);
    BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
    lpItemID := SHBrowseForFolder(BrowseInfo);
    if lpItemId <> nil then begin
        SHGetPathFromIDList(lpItemID, TempPath);
        GlobalFreePtr(lpItemID);
        DocDirectory := TempPath;
        if DocDirectory[Length(DocDirectory)] <> '\' then
            DocDirectory := DocDirectory + '\';
        edDefaultDir.Text := DocDirectory;
    end;
end;

}
