<?
/* Text to Document file conversion library for 3Com PalmPilots
 * Palmlib.php3
 *
 * Copyright (C) 2000 Pierre Dittgen
 * Pierre.Dittgen@bigfoot.com
 *
 * This file is a translation of the txt2pdbdoc tool
 * written in C Paul J. Lucas (plj@best.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as by published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *                         
 * This program is distributed in the hope that it will entertaining,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.   
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

	// Const
	$RECORD_SIZE			= 4096;	// Size of text record
	$PDB_HEADER_SIZE		= 78;	// Size of the file header (don't touch!)
	$PDB_RECORD_HEADER_SIZE	= 8;	// Size of a text record header (don't touch either!)

	/**
	 * Convert a integer value to a two bytes value string
	 */
	function int2($value)
	{
		return sprintf("%c%c", $value / 256, $value % 256);	
	}

	/**
	 * Convert a integer value to a two bytes value string
	 */
	function int4($value)
	{
		for ($i=0; $i<4; $i++)
		{
			$b[$i] = $value % 256;
			$value = (int)(($value - $b[$i]) / 256);
		}
		return sprintf("%c%c%c%c", $b[3], $b[2], $b[1], $b[0]);
	}

	/**
	 * Writes the header of the pdb file containing the title of the document
	 * and different parameters including its size
	 */
	function writeHeader($fd, $title, $content_length)
	{
		global $RECORD_SIZE, $PDB_HEADER_SIZE, $PDB_RECORD_HEADER_SIZE;

		// ============ File header =========================================
		// Title of the document, it's limited to 31 characters
		if (strlen($title) > 31)
			$title = substr($title, 0, 31);
		fwrite($fd, $title);

		// Completion with null '\0' characters
		for ($i=0; $i<32-strlen($title); $i++)
			fwrite($fd, sprintf("%c", 0), 1);
		
		// attributes & version fields
		fwrite($fd, int2(0));
		fwrite($fd, int2(0));

		// create & modify time
		fwrite($fd, sprintf("%c%c%c%c", 6, 209, 68, 174), 4);
		fwrite($fd, sprintf("%c%c%c%c", 6, 209, 68, 174), 4);

		// backup time, modification number, app Info Id, sort info Id
		fwrite($fd, int4(0));
		fwrite($fd, int4(0));
		fwrite($fd, int4(0));
		fwrite($fd, int4(0));

		// Type & creator
		fwrite($fd, 'TEXt', 4);
		fwrite($fd, 'REAd', 4);

		// Id seed & next record list
		fwrite($fd, int4(0));
		fwrite($fd, int4(0));

		// Number of text records
		$full_tr	= (int)($content_length / $RECORD_SIZE);
		$notfull_tr	= $content_length % $RECORD_SIZE;
		$num_records = $full_tr;
		if ($notfull_tr != 0)
			$num_records++;

		// + 1 cause of record 0
		fwrite($fd, int2($num_records + 1));

		// From here...
		$num_offsets = $num_records + 1;
		$offset = $PDB_HEADER_SIZE + $PDB_RECORD_HEADER_SIZE * $num_offsets;
		$index	= 0x40 << 24 | 0x6F8000; // Why not!

		fwrite($fd, int4($offset));
		fwrite($fd, int4($index++));

		$val	= 110 + ($num_offsets - 2) * 8;
		while (--$num_offsets != 0)
		{
			fwrite($fd, int4($val));
			$val += 4096;
			fwrite($fd, int4($index++));
		}
		// To here
		// Don't ask me how it works ;-)

		// ====== Write record 0 ============
		fwrite($fd, int2(1));				// Version
		fwrite($fd, int2(0));				// Reserved1
		fwrite($fd, int4($content_length));	// doc size
		fwrite($fd, int2($num_records));	// num records
		fwrite($fd, int2($RECORD_SIZE));	// record size
		fwrite($fd, int4(0));				// Reserved2
		
	}

	/**
	 * Writes the given text, title on the given file descriptor
	 * Note: It's saved as uncompressed doc
	 * Note2: File descriptor is not closed at the end of the function
	 */
	function writePalmDoc($fd, $content, $title)
	{
		// Write header
		writeHeader($fd, $title, strlen($content));

		// Write content
		fwrite($fd, $content);

		// And flushes all
		flush($fd);
	}


	/**
	 * Creates a palmdoc and force the user to download it
	 */
	function onTheFlyPalmDoc($content, $title, $document)
	{
		// Creates a temp file to put the current doc
		$tempfile = tempnam(".", "palm");

		// Creates the doc from the content
		$fd = fopen($tempfile, 'w');
		writePalmDoc($fd, $content, $title);
		fclose($fd);

		// Forces the download
		header("Content-Type: application/force-download");
		header("Content-Disposition: attachment; filename=\"$document\"");
	
		// And writes the file content on stdout
		$fd = fopen("$tempfile", 'r');
		$content = fread($fd, filesize($tempfile));
		fclose($fd);
		print($content);

		// Cleaning
		unlink($tempfile);
		exit;
	}
?>
