/****************************************************************************
* PalmMOD
*
* Copyright (c) 1998, Intrepid Software Solutions, Inc.
* Distributed under the GNU General Public License.
* See the file "COPYING" included with this distribution.
*
* Revision History:
*
* Rev/date/desc.
* 1.1  August 3, 1998  First public source code release
****************************************************************************/

/* Standard Palm OS includes */

#pragma pack(2)
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

/* GCC hack to fix registers for callback functions */

#include "callback.h"

#define MAXINT (0x7fff)
#define MAXUINT (0xffff)

/* Database definition stuff */

#define FTAppID 'Cks4'
#define FTDBType 'Data'

DmOpenRef FTDB;

typedef struct _InfoRec {
  ULong mode;              /* mode 1 = one data record, to be used over
                                       and over
                              mode 2 = N data records, to be used in order,
                                       then repeated
                              mode 3 = 7 data records, one corresponding to
                                       each day of the week (starting on
                                       Sunday)
                              mode 4 = N data records, one for each day of
                                       the year, by day number
                           */
  SWord last_used;
} InfoRec;

typedef struct _PMPrefs {
  unsigned long signature;
  unsigned short option[16];
} PMPrefs;
#define PREF_SIGNATURE 0x12345678
#define PREF_CURRENT_FONT 0
#define PREF_FIELD_UNDERLINED 1
#define PREF_SAVED_DAY 2

ULong CurrentMode;
Boolean PrefFieldUnderlined = false;
unsigned short PrefSavedDay = 0;
unsigned short CurrentDay = 0;
UInt NumRecords = 0;

DWord ROMVersion;      /* used to decide which SelectDay function to call */
#define V2 0x02000000

char ErrorText[] = "PalmMOD ";

char *dow[7] = {"Sun ", "Mon ", "Tue ", "Wed ", "Thu ", "Fri ", "Sat "};
char days_in_month[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

DateType CurrentDate;     /* today's date */
DateType RecordDate;      /* date of the currently-viewed record */
char date_text[40];       /* holds the displayed date in the title area */

char alert_text[40];      /* holds the text for dynamically-built alerts */

UInt CurrentRecordIndex;        /* the index of the current record */
VoidHand CurrentRecordHandle;   /* the handle of the current text record
                                   the index is derived from unique_id in
                                   the current index record */

FontID CurrentFont = 2;         /* holds the ID of the currently-selected
                                   font */

/* internal function declarations */

static void StopApplication(void);
static int OpenDatabase(void);
static void EventLoop(void);
static int StartApplication(void);
static Boolean FTFrameHandler(EventPtr eventP);
static Boolean FTAboutHandler(EventPtr eventP);
static Boolean FTBadDBHandler(EventPtr eventP);
static Boolean FTPrefsHandler(EventPtr eventP);
static VoidPtr GetObjPtr(Int objectID);
static void SetupScroll(void);
static unsigned int DatesEqual(DateType d1, DateType d2);
static void SetTitle(void);
static UInt FindPreviousRec(void);
static UInt FindNextRec(void);
static UInt FindSpecificRec(DateType pdate);
static UInt PurgeRecords(int days, unsigned char save);
static void SaveCurrentRecord(Boolean del_if_empty);
static void SetupRecord(UInt pindex);
static UInt CreateRecord(DateType pdate);
static void Search (FindParamsPtr findParams);
static void DrawMemoTitle (CharPtr memo, short x, short y, short width);
static void GoToRecord (GoToParamsPtr goToParams, Boolean launchingApp);
static UInt GetRecIndex(DmOpenRef pDB, ULong runique);
static Boolean MySelectDay(SWord *month, SWord *day, SWord *year);
static void MyKeyboardDialog(void);
static void MyGraffitiReference(void);
static void DrawScreen(void);
static void LoadPrefs(void);
static void SavePrefs(void);
static void ExportToMemoPad(void);

/*****************************************************************************
* Function: PilotMain
*
* Description:  Entry point for the application
*
*****************************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{ // PilotMain
int error;

FtrGet(sysFtrCreator, sysFtrNumROMVersion, &ROMVersion);
if (cmd == sysAppLaunchCmdNormalLaunch)
  {
  error = StartApplication();
  if (error)
    return(error);
  EventLoop();
  StopApplication();
  }
else if (cmd == sysAppLaunchCmdFind)
  {
  Search ((FindParamsPtr)cmdPBP);
  }
else if (cmd == sysAppLaunchCmdGoTo)
  {
  // Check if new global variables are set up.  This occurs when the 
  // system intends for the app to start.
  if (launchFlags & sysAppLaunchFlagNewGlobals) 
    {
    // This occurs when the user wants to see a record in this
    // application's database.  The app must be started and 
    // a view to display the data must be opened.
    error = StartApplication ();
    if (error)
      return(error);

    GoToRecord ((GoToParamsPtr) cmdPBP, true);
            
    EventLoop ();
    StopApplication (); 
    }
  else
    {
    // The app was already running.  Open a form to display the record.
    GoToRecord ((GoToParamsPtr) cmdPBP, false);
    }
  }
else if (cmd == sysAppLaunchCmdSaveData)
  {
  // This system routine will save records used by fields and tables.
  // Any additional saving will need to be handled by the application
  // by handling frmSaveEvents sent to each open view from FrmSaveAllForms.
//  FrmSaveAllForms ();
  }
return 0;
} // PilotMain

/*****************************************************************************
* Function:  StartApplication
*
* Description:  Handles any application initialization
*
*****************************************************************************/
static int StartApplication(void)
{ // StartApplication
int error;
DateTimeType ldatetime;
int i;

// Get today's date.
TimSecondsToDateTime (TimGetSeconds (), &ldatetime);
CurrentDate.year  = ldatetime.year - firstYear;
CurrentDate.month = ldatetime.month;
CurrentDate.day   = ldatetime.day;

CurrentDay = 0;
if ((CurrentDate.month == 2) && (CurrentDate.day == 29)) {
  /* February 29 is special */
  CurrentDay = 366;
}
else {
  for (i = 0; i < CurrentDate.month; i++)
    CurrentDay += days_in_month[i];
  CurrentDay += CurrentDate.day;
}

LoadPrefs();
error = OpenDatabase();
if (error)
  FrmGotoForm(3000);
else
  FrmGotoForm(1000);
return 0;
} // StartApplication

/*****************************************************************************
* Function:  OpenDatabase
*
* Description:  Opens/creates the application's database.
*
*****************************************************************************/
static int OpenDatabase(void)
{ // OpenDatabase
/* open or create the database */

FTDB = DmOpenDatabaseByTypeCreator(FTDBType, FTAppID, dmModeReadWrite);
if (!FTDB) {
  return 1;
}


/* search for a record for today */

RecordDate = CurrentDate;
CurrentRecordIndex = CreateRecord(CurrentDate);

return 0;
} // OpenDatabase

/*****************************************************************************
* Function:  EventLoop
*
* Description:  The high-level event processing routine
*
*****************************************************************************/
static void EventLoop(void)
{ // EventLoop
int FormID;
short err;
FormPtr form;
EventType event;

do
  {
  EvtGetEvent(&event, 200);   /* wait 2 seconds for an event */
  if (SysHandleEvent(&event)) continue;
  if (MenuHandleEvent((void *)0, &event, &err)) continue;
  if (event.eType == frmLoadEvent)
    {
    FormID = event.data.frmLoad.formID;
    form = FrmInitForm(FormID);
    FrmSetActiveForm(form);
    switch (FormID)
      {
      case 1000:
        FrmSetEventHandler(form, (FormEventHandlerPtr) FTFrameHandler);
        break;

      case 2000:
        FrmSetEventHandler(form, (FormEventHandlerPtr) FTAboutHandler);
        break;

      case 3000:
        FrmSetEventHandler(form, (FormEventHandlerPtr) FTBadDBHandler);
        break;

      case 5000:
        FrmSetEventHandler(form, (FormEventHandlerPtr) FTPrefsHandler);
        break;
      }
    }
  FrmDispatchEvent(&event);
  } while (event.eType != appStopEvent);

} // EventLoop

/*****************************************************************************
* Function:  StopApplication
*
* Description:  Handles any application cleanup before exiting
*
*****************************************************************************/
static void StopApplication(void)
{ // StopApplication

SavePrefs();
DmCloseDatabase(FTDB);

} // StopApplication

/*****************************************************************************
* Function:  FTFrameHandler
*
* Description:  Handles all event for the journal edit view form
*
*****************************************************************************/
Boolean FTFrameHandler(EventPtr eventP)
{ // FTFrameHandler
int handled = 0;                     // Holds return value
FormPtr form;                        // pointer to the current form
UInt temp_index;
SWord temp_month, temp_day, temp_year;
DateType temp_date;

CALLBACK_PROLOGUE

form = FrmGetActiveForm();
switch (eventP->eType)
  {
  case frmOpenEvent:

    SetupRecord(CurrentRecordIndex);
    FldSetFont(GetObjPtr(1010), CurrentFont);

    SetupScroll();
    SetTitle();

    FrmDrawForm(form);

    handled = 1;
    break;

  case frmGotoEvent:

    SaveCurrentRecord(false);
    SetupRecord(eventP->data.frmGoto.recordNum);

    FldSetScrollPosition(GetObjPtr(1010), eventP->data.frmGoto.matchPos);

    SetupScroll();
    SetTitle();

    FldSetSelection(GetObjPtr(1010),
                    eventP->data.frmGoto.matchPos,
                    eventP->data.frmGoto.matchPos + eventP->data.frmGoto.matchLen);

    FrmDrawForm(form);

    handled = 1;
    break;

  case ctlSelectEvent:
    if (eventP->data.ctlSelect.controlID == 1020)
      {
                                     // Handle Previous button
      SaveCurrentRecord(true);

      if ((temp_index = FindPreviousRec()) < MAXINT) {
        SetupRecord(temp_index);
      }
      else
        SetupRecord(CurrentRecordIndex);

      DrawScreen();

      handled = 1;
      }
    else if (eventP->data.ctlSelect.controlID == 1050)
      {
                                     // Handle Next button
      SaveCurrentRecord(true);
      if ((temp_index = FindNextRec()) < MAXINT) {
        SetupRecord(temp_index);
      }
      else
        SetupRecord(CurrentRecordIndex);

      DrawScreen();

      handled = 1;
      }
    else if (eventP->data.ctlSelect.controlID == 1015) {
                                     // Handle scroll up
      FldScrollField(GetObjPtr(1010), 1, up);
      SetupScroll();
      handled = 1;
    }
    else if (eventP->data.ctlSelect.controlID == 1016) {
                                     // Handle scroll down
      FldScrollField(GetObjPtr(1010), 1, down);
      SetupScroll();
      handled = 1;
    }
    else if (eventP->data.ctlSelect.controlID == 1040) {
                                     // Handle Pick button
      SaveCurrentRecord(true);

      temp_month = RecordDate.month;
      temp_day = RecordDate.day;
      temp_year = RecordDate.year + firstYear;
      if (MySelectDay(&temp_month, &temp_day, &temp_year)) { /* got a new date */
        temp_date.month = temp_month;
        temp_date.day = temp_day;
        temp_date.year = temp_year - firstYear;
        if ((temp_index = FindSpecificRec(temp_date)) < MAXINT) {

          SetupRecord(temp_index);
          DrawScreen();
        }
        else { /* date not found */
          FrmCustomAlert(2000, "Could not create a record for that date", "", "");
        } /* date not found */
      } /* got a new date */
      else {
        SetupRecord(CurrentRecordIndex);
        DrawScreen();
      }
    handled = 1;
    }
    else if (eventP->data.ctlSelect.controlID == 1060) {
                                     // Handle font 0 pushbutton
      FldSetFont(GetObjPtr(1010), 0);
      CurrentFont = 0;
      SetupScroll();
      handled = 1;
    }
    else if (eventP->data.ctlSelect.controlID == 1070) {
                                     // Handle font 1 pushbutton
      FldSetFont(GetObjPtr(1010), 1);
      CurrentFont = 1;
      SetupScroll();
      handled = 1;
    }
    else if (eventP->data.ctlSelect.controlID == 1080) {
                                     // Handle font 2 pushbutton
      FldSetFont(GetObjPtr(1010), 2);
      CurrentFont = 2;
      SetupScroll();
      handled = 1;
    }
    break;

  case keyDownEvent:
    if (eventP->data.keyDown.chr == pageUpChr) {
      FldScrollField(GetObjPtr(1010), 10, up);
      handled = 1;
    }
    else if (eventP->data.keyDown.chr == pageDownChr) {
      FldScrollField(GetObjPtr(1010), 10, down);
      handled = 1;
    }
    SetupScroll();
    break;

  case menuEvent:
    switch (eventP->data.menu.itemID)
      {
      case 2020:                          // Copy
        FldCopy(GetObjPtr(1010));
        handled = 1;
        break;

      case 2040:                          // Select All
        FldSetSelection(GetObjPtr(1010), 0, FldGetTextLength(GetObjPtr(1010)));
        handled = 1;
        break;

      case 2050:                         // Previous date
        SaveCurrentRecord(true);

        if ((temp_index = FindPreviousRec()) < MAXINT) {
          SetupRecord(temp_index);
        }
        else
          SetupRecord(CurrentRecordIndex);

        DrawScreen();

        handled = 1;
        break;

      case 2080:                         // Next date
        SaveCurrentRecord(true);

        if ((temp_index = FindNextRec()) < MAXINT) {
          SetupRecord(temp_index);
        }
        else
          SetupRecord(CurrentRecordIndex);

        DrawScreen();

        handled = 1;
        break;

      case 2060:                         // Other date
      
        SaveCurrentRecord(true);
        if ((CurrentMode == 3) || (CurrentMode == 4)) {
          temp_month = RecordDate.month;
          temp_day = RecordDate.day;
          temp_year = RecordDate.year + firstYear;
          if (MySelectDay(&temp_month, &temp_day, &temp_year)) { /* got a new date */
            temp_date.month = temp_month;
            temp_date.day = temp_day;
            temp_date.year = temp_year - firstYear;
            if ((temp_index = FindSpecificRec(temp_date)) < MAXINT) {

              SetupRecord(temp_index);
              DrawScreen();
            }
            else { /* date not found */
              FrmCustomAlert(2000, "Could not create a record for that date", "", "");
            } /* date not found */
          } /* got a new date */
          else {
            SetupRecord(CurrentRecordIndex);
            DrawScreen();
          }
        }
        else {
          FrmCustomAlert(3000, "This function is not", " applicable to the", " current database.");
          SetupRecord(CurrentRecordIndex);
          DrawScreen();
        }
        
        handled = 1;
        break;

      case 2070:                        // About...
        FrmPopupForm(2000);
        handled = 1;
        break;

      case 5000:                        // Preferences...
        SaveCurrentRecord(true);
        FrmGotoForm(5000);
        handled = 1;
        break;

      case 2099:                        // Export To MemoPad
        ExportToMemoPad();
        handled = 1;
        break;
      }

  case fldChangedEvent:
    SetupScroll();
    FrmDrawForm(form);
    handled = 1;
    break;

  case nilEvent:
    handled = 1;
    break;
  }

CALLBACK_EPILOGUE

return handled;
} // FTFrameHandler

/*****************************************************************************
* Function:  ACAboutHandler
*
* Description:  Handles all event for the about form
*
*****************************************************************************/
Boolean FTAboutHandler(EventPtr eventP)
{ // FTAboutHandler
int handled = 0;                     // Holds return value

CALLBACK_PROLOGUE

switch (eventP->eType)
  {
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = 1;
    break;
  case ctlEnterEvent:
    if (eventP->data.popSelect.controlID == 2010)
      {
      FrmReturnToForm(1000);
      handled = 1;
      }
    break;
  }

CALLBACK_EPILOGUE

return(handled);
} // FTAboutHandler

/*****************************************************************************
* Function:  FTBadDBHandler
*
* Description:  Handles all events for the purge form
*
*****************************************************************************/
Boolean FTBadDBHandler(EventPtr eventP)
{ // FTBadDBHandler
int handled = 0;                     // Holds return value
FormPtr form;                        // pointer to the current form

CALLBACK_PROLOGUE

form = FrmGetActiveForm();
switch (eventP->eType)
  {
  case frmOpenEvent:
    FrmDrawForm(form);
    handled = 1;
    break;
  }

CALLBACK_EPILOGUE

return(handled);
} // FTBadDBHandler

/*****************************************************************************
* Function:  FTPrefsHandler
*
* Description:  Handles all events for the preferences form
*
*****************************************************************************/
Boolean FTPrefsHandler(EventPtr eventP)
{ // FTPrefsHandler
int handled = 0;                     // Holds return value
FormPtr form;                        // pointer to the current form

CALLBACK_PROLOGUE

form = FrmGetActiveForm();
switch (eventP->eType)
  {
  case frmOpenEvent:
    if (PrefFieldUnderlined)
      CtlSetValue(GetObjPtr(5030), 1);
    else
      CtlSetValue(GetObjPtr(5030), 0);
    FrmDrawForm(form);
    handled = 1;
    break;

  case ctlSelectEvent:
    if (eventP->data.ctlSelect.controlID == 5998) // OK button
      {
      if (CtlGetValue(GetObjPtr(5030)))
        PrefFieldUnderlined = true;
      else
        PrefFieldUnderlined = false;
      SavePrefs();
      FrmGotoForm(1000);
      handled = 1;
      }
    else if (eventP->data.ctlSelect.controlID == 5999) // Cancel button
      {
      FrmGotoForm(1000);
      handled = 1;
      }
    break;
  }

CALLBACK_EPILOGUE

return(handled);
} // FTPrefsHandler

/*****************************************************************************
* Function:    GetObjPtr
*
* Description: Gets a pointer to an object
*
* Parms:       objectID - the ID of the object, as specified in the .rc file
*
* Returns:     a pointer to the specified objec
*****************************************************************************/
static VoidPtr GetObjPtr(Int objectID)
{
  FormPtr frm;

  frm = FrmGetActiveForm();
  return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

/*****************************************************************************
* Function:    MySelectDay
*
* Description: Calls the correct SelectDay function based on the ROM version
*
*****************************************************************************/
static Boolean MySelectDay(SWord *month, SWord *day, SWord *year)
{
SWord temp_month = *month;
SWord temp_day   = *day;
SWord temp_year  = *year;
Boolean result   = false;

if (ROMVersion < V2) {       // version 1.0 ROM, Pilot 1000 and 5000
  result = SelectDayV10(&temp_month, &temp_day, &temp_year, "Pick Journal Date");
}
else {                       // more modern ROM version
  result = SelectDay(selectDayByDay,
                     &temp_month,
                     &temp_day,
                     &temp_year,
                     "Pick Journal Date");
}
if (result) {
  *month = temp_month; *day = temp_day; *year = temp_year;
}

return (result);
}

/*****************************************************************************
* Function:    SetupScroll
*
* Description: Sets up the field scroll buttons on the main edit view form.
*              *** ALSO *** sets up the font selector pushbuttons based on
*              the font currently in use.
*
*****************************************************************************/
static void SetupScroll(void)
{
FieldAttrType   attr;

if (FldScrollable(GetObjPtr(1010), up))
  CtlSetLabel(GetObjPtr(1015), "\001");
else
  CtlSetLabel(GetObjPtr(1015), "\003");
if (FldScrollable(GetObjPtr(1010), down))
  CtlSetLabel(GetObjPtr(1016), "\002");
else
  CtlSetLabel(GetObjPtr(1016), "\004");

CtlSetValue(GetObjPtr(1060), FldGetFont(GetObjPtr(1010)) == 0);
CtlSetValue(GetObjPtr(1070), FldGetFont(GetObjPtr(1010)) == 1);
CtlSetValue(GetObjPtr(1080), FldGetFont(GetObjPtr(1010)) == 2);
if ((CurrentMode != 3) && (CurrentMode != 4)) {
  CtlHideControl(GetObjPtr(1040));

FldGetAttributes (GetObjPtr(1010), &attr);
if (PrefFieldUnderlined)
  attr.underlined = true;
else
  attr.underlined = false;
FldSetAttributes (GetObjPtr(1010), &attr);
}
}

/*****************************************************************************
* Function:    DatesEqual
*
* Description: Tests two dates (DateType) for equality
*
*****************************************************************************/
static unsigned int DatesEqual(DateType d1, DateType d2)
{
if ((d1.year == d2.year) &&
    (d1.month == d2.month) &&
    (d1.day == d2.day))
  return(true);
else
  return(false);
}

/*****************************************************************************
* Function:    SetTitle
*
* Description: Sets up the edit view title based on the currently-displayed
*              record's date.
*
*****************************************************************************/
static void SetTitle()
{
if ((CurrentMode == 3) || (CurrentMode == 4)) {
  StrCopy(date_text,
          dow[DayOfWeek(RecordDate.month, RecordDate.day, RecordDate.year + firstYear)]);
  DateToAscii(RecordDate.month,
              RecordDate.day,
              RecordDate.year + firstYear,
              dfMDYLongWithComma,
              &date_text[StrLen(date_text)]);

  FldSetTextPtr(GetObjPtr(1030), date_text);
}
}

/*****************************************************************************
* Function:    FindPreviousRec
*
* Description: Searches the database for a record with the latest date less
*              than (before) the currently-displayed record.
*
* Returns:     the index of the index record if found, or
*              MAXINT if no record was found
*****************************************************************************/
static UInt FindPreviousRec(void)
{
ULong temp_long;

switch (CurrentMode) {
  case 1:
    return(CreateRecord(RecordDate));
    break;
    
  case 2:
    if (CurrentRecordIndex > 1)
      return(CurrentRecordIndex - 1);
    else
      return(NumRecords - 1);
    break;
    
  case 3:
  case 4:
    temp_long = DateToDays(RecordDate);
    if (temp_long > 0)
      temp_long--;
    DateDaysToDate(temp_long, &RecordDate);
    return(CreateRecord(RecordDate));
    break;
}
}

/*****************************************************************************
* Function:    FindNextRec
*
* Description: Searches the database for a record with the earliest date
*              greater than (after) the currently-displayed record.
*
* Returns:     the index of the index record if found, or
*              MAXINT if no record was found
*****************************************************************************/
static UInt FindNextRec(void)
{
ULong temp_long;

switch (CurrentMode) {
  case 1:
  case 2:
    return(CreateRecord(RecordDate));
    break;
    
  case 3:
  case 4:
    temp_long = DateToDays(RecordDate);
    temp_long++;
    DateDaysToDate(temp_long, &RecordDate);
    return(CreateRecord(RecordDate));
    break;
}
}

/*****************************************************************************
* Function:    FindSpecificRec
*
* Description: Searches the database for a record with the specified date.
*
* Returns:     the index of the index record if found, or
*              MAXINT if no record was found
*****************************************************************************/
static UInt FindSpecificRec(DateType pdate)
{
RecordDate = pdate;
return(CreateRecord(RecordDate));
}

/*****************************************************************************
* Function:    SaveCurrentRecord
*
* Description: Saves the record indicated by CurrentRecordIndex
*
*****************************************************************************/
static void SaveCurrentRecord(Boolean del_if_empty)
{
  FldSetTextHandle(GetObjPtr(1010), NULL);
}

/*****************************************************************************
* Function:    SetupRecord
*
* Description: Sets up CurrentRecordHandle with the text record specified in
*              the passed index record. The specified index record MUST point
*              to a valid record (via unique_id) or a fatal error is
*              generated. CurrentRecordHandle is set upon exit.
*
*****************************************************************************/
static void SetupRecord(UInt pindex)
{
VoidHand RecHandle;
CharPtr temp_pointer;

CurrentRecordIndex = pindex;

CurrentRecordHandle = DmQueryRecord(FTDB, CurrentRecordIndex);
ErrFatalDisplayIf(!CurrentRecordHandle, ErrorText);
temp_pointer = MemHandleLock(CurrentRecordHandle);

FldSetText (GetObjPtr(1010),
            CurrentRecordHandle,
            0,
            StrLen(temp_pointer)+1);
MemHandleUnlock (CurrentRecordHandle);
CurrentRecordHandle = 0;
}

/*****************************************************************************
* Function:    DrawScreen
*
* Description: Handles all the details for re-drawing the edit view:
*              - sets up scroll arrows
*              - sets up title
*              - draws the form
*              - sets the field focus
*
*****************************************************************************/
static void DrawScreen(void)
{
FormPtr form;

form = FrmGetActiveForm();
SetupScroll();
SetTitle();
FrmDrawForm(form);
}

/*****************************************************************************
* Function:    CreateRecord
*
* Description: Makes a new pair of journal entry records (index record and
*              text record) identified by the passed date.
*
*****************************************************************************/
static UInt CreateRecord(DateType pdate)
{
UInt index = MAXINT;
VoidHand RecHandle, RecHandle2;
CharPtr motdRecPointer;
InfoRec *InfoRecPointer;
InfoRec lInfoRec;
int i;
static Boolean first_time = true;

  /* check for a message-of-the-day database and grab the correct template
     text if available */

  if (FTDB) {
      NumRecords = DmNumRecords(FTDB);
      RecHandle = DmGetRecord(FTDB, 0);
      if (RecHandle) { /* good info record */

        InfoRecPointer = MemHandleLock(RecHandle);
        MemMove(&lInfoRec, InfoRecPointer, sizeof(InfoRec));

        CurrentMode = lInfoRec.mode;
        switch (lInfoRec.mode) {
          case 1:
            index = 1;
            break;

          case 2:
            if (first_time && (CurrentDay == PrefSavedDay)) {
              index = lInfoRec.last_used;
              first_time = false;
            }
            else if (lInfoRec.last_used >= (DmNumRecords(FTDB) - 1))
              index = 1;
            else
              index = lInfoRec.last_used + 1;
            break;

          case 3:
            if (DmNumRecords(FTDB) < 8)
              index = 0;
            else
              index = DayOfWeek(pdate.month, pdate.day, pdate.year + firstYear) + 1;
            break;

          case 4:
            if (DmNumRecords(FTDB) < 367)
              index = 0;
            else {
              index = 0;
              if ((pdate.month == 2) && (pdate.day == 29)) {
                /* February 29 is special */
                index = 366;
              }
              else {
                for (i = 0; i < pdate.month; i++)
                  index += days_in_month[i];
                index += pdate.day;
              }
            }
            break;

          default:
            index = 0;
            break;
        } /* switch */

        lInfoRec.last_used = index;
        DmWrite(InfoRecPointer, 0, &lInfoRec, sizeof(InfoRec));
        MemPtrUnlock(InfoRecPointer);
        DmReleaseRecord(FTDB, 0, true);

      } /* good info record */

  }
  
  return(index);
}


/***********************************************************************
 *
 * FUNCTION:    Search
 *
 * DESCRIPTION: This routine searchs the database for records 
 *              containing the string passed.
 *
 ***********************************************************************/
static void Search (FindParamsPtr findParams)
{
Word pos;
UInt recordNum;
VoidHand recordH;
RectangleType r;
Boolean done;
Boolean match;
DmOpenRef dbP;
UInt cardNo = 0;
LocalID dbID;
CharPtr testpointer;
char temp_text[20], temp_text2[10];

// Find the application's data file.
dbP = DmOpenDatabaseByTypeCreator(FTDBType, FTAppID, findParams->dbAccesMode);
if (!dbP)
  {
  findParams->more = false;
  return;
  }
DmOpenDatabaseInfo(dbP, &dbID, 0, 0, &cardNo, 0);

// Display the heading line.
done = FindDrawHeader (findParams, "PalmMOD");
if (done) goto Exit;
    
// Search the memos for the "find" string.
recordNum = findParams->recordNum;
while (true)
  { /* while */
  recordH = DmQueryRecord(dbP, recordNum);

  // Have we run out of records?
  if (! recordH)
    {
    findParams->more = false;           
    break;
    }

  testpointer = MemHandleLock (recordH);
         
// Search for the string passed,  if it's found display the title
// of the memo.

  match = FindStrInStr (testpointer, findParams->strToFind, &pos);
  if (match)
    { /* match */
    // Add the match to the find paramter block,  if there is no room to
    // display the match the following function will return true.
    done = FindSaveMatch (findParams, recordNum, pos, 0, 0, cardNo, dbID);

    if (! done)
      {
      // Get the bounds of the region where we will draw the results.
      FindGetLineBounds (findParams, &r);

      // Display the title of the description.
      StrCopy(temp_text, "Record ");
      StrCat(temp_text, StrIToA(temp_text2, recordNum));
      DrawMemoTitle (temp_text, r.topLeft.x+1, r.topLeft.y, r.extent.x-2);

      findParams->lineNumber++;
      }
    } /* match */

  if (done) break;

  MemHandleUnlock(recordH);
  recordH = 0;
  recordNum++;

  } /* while */

if (recordH)
  MemHandleUnlock(recordH);
        
Exit:
    DmCloseDatabase (dbP);  
}


/***********************************************************************
 *
 * FUNCTION:    DrawMemoTitle
 *
 * DESCRIPTION: This routine draws the title of the specified memo. 
 *
 ***********************************************************************/
static void DrawMemoTitle (CharPtr memo, short x, short y, short width)
{
    Int titleLen;
    Word charsToDraw;
    short titleWidth;
    CharPtr ptr;
    Boolean stringFit;
    

    ptr = StrChr (memo, linefeedChr);
    if (ptr)
        charsToDraw = (Word) (ptr - memo);
    else
        charsToDraw = StrLen (memo);

    titleWidth = width;
    titleLen = charsToDraw;
    FntCharsInWidth (memo, &titleWidth, &titleLen, &stringFit);

    if (stringFit)
        {
        WinDrawChars (memo, titleLen, x, y);
        }
    else
        {
        width -= (FntCharWidth ('.') * 3);
        while (titleWidth > width || 
            memo[titleLen - 1] == ' ' || 
            memo[titleLen - 1] == tabChr)
            {
            titleWidth -= FntCharWidth (memo[--titleLen]);
            }
        WinDrawChars (memo, titleLen, x, y);
        x += titleWidth;
        WinDrawChars ("...", 3, x, y);
        }
}


/***********************************************************************
 *
 * FUNCTION:    GoToRecord
 *
 ***********************************************************************/
static void GoToRecord (GoToParamsPtr goToParams, Boolean launchingApp)
{
Word recordNum;
UInt attr;
ULong uniqueID;
EventType event;

recordNum = goToParams->recordNum;
        
// Send events to load and goto a form selecting the matching text.
MemSet (&event, sizeof(EventType), 0);
 
event.eType = frmGotoEvent;
event.data.frmGoto.recordNum = recordNum;
event.data.frmGoto.matchPos = goToParams->matchPos;
event.data.frmGoto.matchLen = goToParams->searchStrLen;
event.data.frmGoto.matchFieldNum = goToParams->matchFieldNum;
event.data.frmGoto.formID = 1000;
EvtAddEventToQueue (&event);
}

/*****************************************************************************
* Function:    LoadPrefs
*
* Description: Reads PalmMOD preferences
*****************************************************************************/
static void LoadPrefs(void)
{
SWord prefsVersion = 0;
PMPrefs prefs;
Word prefsSize;

prefsSize = sizeof (prefs);
if (ROMVersion >= V2)
  prefsVersion = PrefGetAppPreferences (FTAppID, 1, &prefs, &prefsSize, true);
else
  prefsVersion = PrefGetAppPreferencesV10 (FTAppID, 1, &prefs, prefsSize);
if ((prefsVersion > noPreferenceFound) && (prefs.signature == PREF_SIGNATURE)) {
  CurrentFont         = prefs.option[PREF_CURRENT_FONT];
  PrefFieldUnderlined = prefs.option[PREF_FIELD_UNDERLINED];
  PrefSavedDay        = prefs.option[PREF_SAVED_DAY];
}
}

/*****************************************************************************
* Function:    SavePrefs
*
* Description: Writes PalmMOD preferences
*****************************************************************************/
static void SavePrefs(void)
{
PMPrefs prefs;

MemSet(&prefs, sizeof(prefs), 0);

prefs.signature                     = PREF_SIGNATURE;
prefs.option[PREF_CURRENT_FONT]     = CurrentFont;
prefs.option[PREF_FIELD_UNDERLINED] = PrefFieldUnderlined;
prefs.option[PREF_SAVED_DAY]        = CurrentDay;
if (ROMVersion >= V2)
  PrefSetAppPreferences (FTAppID, 1, 1, &prefs, sizeof(prefs), true);
else
  PrefSetAppPreferencesV10 (FTAppID, 1, &prefs, sizeof(prefs));
}

/*****************************************************************************
* Function:    ExportToMemoPad
*
* Description: Makes a new MemoPad record consisting of a PalmJournal
*              headling and the contents of the current PalmJournal record.
*****************************************************************************/
static void ExportToMemoPad(void)
{
  DmOpenRef MemoDB;
  UInt index = 0;
  VoidHand RecHandle;
  Ptr RecPointer;
  ULong memolen;
  ULong errorlen, datelen;
  char LF = '\012';

  errorlen = StrLen(ErrorText);
  datelen = StrLen(date_text);
  memolen = FldGetTextLength(GetObjPtr(1010));
  MemoDB = DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
  if (MemoDB) {
      RecHandle = DmNewRecord(MemoDB, &index, errorlen + datelen + memolen + 2);
      RecPointer = MemHandleLock(RecHandle);
      DmWrite(RecPointer, 0, ErrorText, errorlen);
      DmWrite(RecPointer, errorlen, date_text, datelen);
      DmWrite(RecPointer, errorlen + datelen, &LF, 1);
      DmWrite(RecPointer,
              errorlen + datelen + 1,
              FldGetTextPtr(GetObjPtr(1010)),
              memolen + 1);
      MemPtrUnlock(RecPointer);
      DmReleaseRecord(MemoDB, index, true);
      DmCloseDatabase(MemoDB);
  }
}
