/*
 * Program:	Exclusive create of a file, NFS kludge version
 *
 * Author:	Mark Crispin
 *		Networks and Distributed Computing
 *		Computing & Communications
 *		University of Washington
 *		Administration Building, AG-44
 *		Seattle, WA  98195
 *		Internet: MRC@CAC.Washington.EDU
 *
 * Date:	17 December 1999
 * Last Edited:	17 December 1999
 *
 * Copyright 1999 by the University of Washington
 *
 *  Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies and that both the
 * above copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the University of Washington not be
 * used in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  This software is made available
 * "as is", and
 * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
 * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */


/* SUN-OS had an NFS, As kludgy as an albatross;
 * And everywhere that it was installed, It was a total loss.
 *  -- MRC 9/25/91
 */

/* Exclusive create of a file
 * Accepts: file name
 * Return: T if success, NIL if failed, -1 if retry
 */

long crexcl (char *name)
{
  long ret = -1;
  int i;
  char hitch[MAILTMPLEN];
  struct stat sb;
				/* build hitching post file name */
  sprintf (hitch,"%s.%lu.%d.",name,(unsigned long) time (0),getpid ());
  i = strlen (hitch);		/* append local host name */
  gethostname (hitch + i,(MAILTMPLEN - i) - 1);
				/* try to get hitching-post file */
  if ((i = open (hitch,O_WRONLY|O_CREAT|O_EXCL,(int) lock_protection)) >= 0) {
    close (i);			/* close the hitching-post */
    link (hitch,name);		/* tie hitching-post to lock, ignore failure */
				/* success if link count now 2 */
    if (!stat (hitch,&sb) && (sb.st_nlink == 2)) ret = LONGT;
    unlink (hitch);		/* flush hitching post */
  }
				/* fail unless error is EEXIST */
  else if (errno != EEXIST) ret = NIL;
  return ret;
}
