/*
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License. 
 * 
 * The Original Code is pilot-mailsync.
 * 
 * The Initial Developer of the Original Code is Hollis Blanchard
 * <hollis@andrew.cmu.edu>. Portions created by Hollis Blanchard are Copyright
 * (C) 2000 Hollis Blanchard. All Rights Reserved.
 * 
 * Contributor(s):
 * Frederic Devernay <Frederic.Devernay@sophia.inria.fr>
 * 
 */

#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <pi-socket.h>
#include <pi-dlp.h>

#include "util.h"


extern time_t parsedate(const char *d); /* from parsedate.y */

void Disconnect(int pilot_socket)
{
	if (pilot_socket == 0)
		return;

	dlp_EndOfSync(pilot_socket, 0);
	pi_close(pilot_socket);
	pilot_socket = 0;
}

int Connect(char *device)
{
	struct pi_sockaddr addr;
	int ret, pilot_socket;

	if (0 == (pilot_socket = pi_socket(PI_AF_SLP,PI_SOCK_STREAM,PI_PF_PADP))) {
		perror("pi_socket");
		exit(1);
	}

	addr.pi_family = PI_AF_SLP;
	strcpy(addr.pi_device, device);
  
	ret = pi_bind(pilot_socket, (struct sockaddr*)&addr, sizeof(addr));
	if (ret == -1) {
		fprintf(stderr, "Unable to bind to port '%s'.\n", device);
		exit(1);
	}
	
	fprintf(stderr,
			"Waiting for connection on %s (press the HotSync button now)...\n",
			device);

	ret = pi_listen(pilot_socket, 1);
	if (ret == -1) {
		perror("pi_listen");
		exit(1);
	}

	pilot_socket = pi_accept(pilot_socket, 0, 0);
	if (pilot_socket == -1) {
		perror("pi_accept");
		exit(1);
	}

	fprintf(stderr, "Connected.\n");
  
	return pilot_socket;
}


/* convert uid to username */
char * getname(uid_t uid)
{
	struct passwd *pw;

	if ((pw = getpwuid(uid)) == NULL)
		return NULL;
	return pw->pw_name;
}

struct tm *DateToTm(const char *datestr)
{
	time_t parsedtime;

	parsedtime = parsedate(datestr); /* from parsedate.y */
	return localtime(&parsedtime);
}

char *strip_cr(const char *orig)
{
	char *stripped;
	const char *p1=orig;
	char *p2;

	if (orig == NULL)
		return NULL;

	stripped = (char *)malloc(strlen(orig));
	p2 = stripped;

	/* copy everything that isn't CR */
	while (*p1 != '\0') {
		if (*p1 != '\r') {
			*p2++ = *p1;
		}
		p1++;
	}
	*p2 = '\0'; /* terminate nicely */

	return stripped;
}

