#!/usr/bin/perl

# pilot2pine.pl v0.1  12/23/98
# by Mike Hughes, wxh@altonet.com
# Released under the GNU General Public License

# modified by Ralph Bodenner, rkb@duke.edu
# *** 1999-01-15 changes:
# - create semirandom filename for temp file
# for Pilot->pine conversion:
# - list names in pine addressbook as "Last, First Middle", or "Business" if
#      Pilot name fields empty 
# - nickname in pine = FirstmiddleLast from Pilot fields if either of those 
#      fields not empty, otherwise
#   nickname in pine = Businessname from Pilot field

# I have no experience at all with making a arg checker, so here goes!
# I get this feeling that this is horribly inefficient... (and wrong)

if ($ARGV[2] && $ARGV[1] && $ARGV[0]) {
   if ($ARGV[0] eq "-r") {
      $export = $ARGV[2];
      $import = $ARGV[1];
      @command = "/bin/touch $export";
      system @command;
      open(EXPORT, "> $export");
      &parse_pine_addresses;
      print "Done converting. Pilot addressbook saved as $export\n";
   }

   else { 
      &print_info;
   }
}

elsif ($ARGV[1] && $ARGV[0]) {

   $pilotdownload = &rand_tmp_filename;

   if ($ARGV[0] eq "-n") {
      $export = $ARGV[1];
      &get_pilot_addresses;
      @command = "/bin/touch $export";
      system @command;
      open(EXPORT, ">$export");
      &parse_pilot_addresses;
      print "Found and skipped $missed_count entries with no email addresses.\n";
      print "Found and substituted $alt_nick_count missing nicknames.\n";
      print "Done writing new addressbook to $export\n";
   }

   elsif ($ARGV[0] eq "-a") {
      $export = $ARGV[1];
      &get_pilot_addresses;
      @command = "/bin/touch $export";
      system @command;
      open(EXPORT, ">>$export");
      &parse_pilot_addresses;
      print "Found and skipped $missed_count entries with no email addresses.\n";
      print "Found and substituted $alt_nick_count missing nicknames.\n";
      print "Done appending addressbook to $export\n";
   }
 
   else {
      &print_info;
   }
}

else {
   &print_info;
}

# delete temp file
unlink $pilotdownload;

# make up a semirandom temp file name
sub rand_tmp_filename {
   $rnum = rand 1;
   $num = substr($rnum, 3, 4);
   return "/tmp/temp" . $num . ".p2p"
}

sub get_pilot_addresses {
   print "Getting Palm Pilot addressbook...\n";
   @get = "pilot-addresses -p /dev/pilot -w " . $pilotdownload;
   system @get;
}
   
sub parse_pilot_addresses {

   # Explanation of pilot array
   # $address_line[0] = Last Name
   # $address_line[1] = First Middle Names
   # $address_line[2] = Title (Unused)
   # $address_line[3] = Company (Used if Name fields are empty)
   # $address_line[4] = Work Phone (Unused)
   # $address_line[5] = Home Phone (Unused)
   # $address_line[6] = Fax Number (Unused)
   # $address_line[7] = Other Notes
   # $address_line[8] = E-mail address

   open(PILOTADDRS, $pilotdownload);
   @pilot_addresses = <PILOTADDRS>;
   close PILOTADDRS;
   $missed_count = 0; $alt_nick_count = 0;

   foreach $pilot_address (@pilot_addresses) {
      @address_line = split /","/, $pilot_address;

      foreach $address_field (@address_line) {
         $address_field =~ s/""/"/g;
      }

      if ($address_line[8] ne '') { 
         $address_line[0] =~ tr/^"//d;
         $address_line[$#address_line] =~ tr/"$//d;

         $first_middle_with_spaces = $address_line[1];
         $address_line[1] =~ tr/ ,//d;
         $last_name_no_spaces = $address_line[0];
         $last_name_no_spaces =~ tr/ //d;

         # failing that, use business name
         if (($address_line[0] eq '') && ($address_line[1] eq '')) {
            $business_with_spaces = $address_line[3];
            $address_line[3] =~ tr/ ,//d;
            print EXPORT "$address_line[3]\t";
            $alt_nick_count++;
         }
         # use FirstmiddleLast as nickname
         else {
            print EXPORT "$address_line[1]"."$last_name_no_spaces\t";   
         }

         # Last, First Middle ...  (Full Name Field)
         # ... or Business (In place of Full Name Field)
         if (($address_line[0] eq '') && ($address_line[1] eq '')) {
           print EXPORT "$business_with_spaces\t";
           $used_business = 1;
         }
         else {
            print EXPORT "$first_middle_with_spaces $address_line[0]\t";
            $used_business = 0;
         }
         # Email
         # can't have commas here, should only happen in firstmiddle
         $first_middle_with_spaces =~ tr/,//d;
         if ($used_business == 0) {
            print EXPORT "$first_middle_with_spaces $address_line[0] <$address_line[8]>\t";
            print EXPORT "\t"; 
         }
         elsif ($used_business == 1) {
            print EXPORT "$business_with_spaces <$address_line[8]>\t";
            print EXPORT "\t";
         }

         # Other Notes = phone number from Pilot
         print EXPORT "$address_line[7]\t\n"; 
      }
      else { 
        $missed_count++;
      }
   }

   close EXPORT;
}

sub parse_pine_addresses {
   open(PINEADDRS, $import);
   @pine_addresses = <PINEADDRS>;
   close(PINEADDRS);

   # Explanation of current array:
   # @address_line[0] - FirstMiddle (Nickname)
   # @address_line[1] - First Middle Last (Name)
   # @address_line[2] - First Middle Last <email@host.com> (email)
   # @address_line[3] -  (blank for Fcc)
   # @address_line[4] - Other (Other...)

   foreach $pine_address (@pine_addresses) {
      @address_line = split/\t/, $pine_address;
      # split firstmiddle and last names up...

      $address_line[1] =~ /.+ ([^ ]+)/;
      $last_name = $1;
      $address_line[1] =~ /(.+) [^ ]+/;
      $firstmiddle_name = $1;
 
      if (grep(/</, $address_line[2]) eq '1') { # is the address in name <email> format?
         $address_line[2] =~ /<(.+)>/;
         $email_address = $1;
      } 

      else {
         $email_address = $address_line[2];
      }

      print EXPORT "\"$last_name\",";
      print EXPORT "\"$firstmiddle_name\",";
      print EXPORT "\"\",\"\",\"\",\"\",\"\",";
      print EXPORT "\"$address_line[4]\",";
      print EXPORT "\"$email_address\",";
      print EXPORT "\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",";
      print EXPORT "\"0\"\n";

   }    # end of foreach $pine_address (@pine_addresses)
}       # end of subroutine

sub print_info {
   print "Copyright (C) 1998 Mike Hughes.  Released under the GNU GPL.\n\n";
   print "Converts Palm Pilot addressbooks to pine addressbooks. Usage:\n\n";
   print " pilot2pine.pl [-n|-a|-r] file1 [file2]\n\n";
   print " -n file1       = writes new addressbook to <file1>\n";
   print " -a file1       = appends addresses to <file1>\n";
   print " -r file1 file2 = converts a pine addressbook <file1> to a pilot\n";
   print "                  address book <file2>\n\n";
   exit;
}

