/*
 * syncal_cat.c
 * syncal Version 0.7.2
 *
 * Copyright (c) 1999, John Franks
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <ctype.h>
#include "syncal.h"



char
	pilot_cats[MAX_PILOT_CATS][16],
	archv_cats[MAX_PILOT_CATS][16];


void
get_pcats( struct AppointmentAppInfo *	ainfo_p)
{
	char *cp;
	int i;

	for ( i = 0; i < MAX_PILOT_CATS; i++ ) {
		if ( ainfo_p->category.name[i][0]) {
			mystrncpy( pilot_cats[i], ainfo_p->category.name[i], 16);
			cp = pilot_cats[i];
			while ( *cp ) {
				if ( !isalnum( *cp) && (*cp != '.')
				     		&& (*cp != '-'))
					*cp = '_';
				cp++;
			}
			if ( i > 0 )
				use_datebk3 = TRUE;
			if ( archv_cats[i][0] == '\0') {
				mystrncpy( archv_cats[i], pilot_cats[i], 16);
			}
		}
		else {
			pilot_cats[i][0] = '\0';
		}
	}
}

void
get_arcats( char *buf)
{
	char
		*cp,
		*cp2;
	int
		i;

	for ( i = 0; i < MAX_PILOT_CATS; i++ ) {
		archv_cats[i][0] = '\0';
	}

	if ( (cp = strchr( buf, ']')) == NULL)
		format_error( "Missing ']' in archive category list");
	else
		*cp = '\0';

	if ( (cp = strchr( buf, '[')) == NULL )
		format_error( "Missing '[' in archive category list");
	cp++;

	cp = skip( cp);
	while ( cp && *cp) {
		if ( !isdigit( *cp))
			format_error( "Error in archive category list");
		i = atoi( cp);
		while ( cp && !isspace( *cp))
			cp++;
		cp = skip( cp);
		cp2 = cp;
		while ( !isspace( *cp2))
			cp2++;
		if ( *cp2) {
			*cp2 = '\0';
			mystrncpy( archv_cats[i], cp, 16);
			cp = cp2 + 1;
		}
		else {
			mystrncpy( archv_cats[i], cp, 16);
			cp = cp2;
		}
		cp = skip( cp);
	}
}

static void
write_category( FILE *fp)
{
	int i;

	if ( !use_datebk3)
		return;

	fprintf( fp, "PilotCategories [\n");
	for ( i = 0; i < MAX_PILOT_CATS; i++) {
		if ( pilot_cats[i][0] )
			fprintf( fp, " %d %s\n", i, pilot_cats[i]);
	}
	fprintf( fp, "]\n");
}

void
mem2ical( struct A_list *lptr, FILE *fp )
{
	FILE *cfp;
	char
		*cp,
		ent_buf[MIDLEN],
		fname[2*SMALLLEN],
		backname[2*SMALLLEN];
	mode_t
		default_mode;
	struct stat
		stat_buf;

	struct A_entry  *ptr;
	int i;

	if ( !no_action) {
		fprintf( fp, "Calendar [v2.0]\n");
		write_category( fp);
	}
	ptr = lptr->first;
	while ( ptr != NULL ) {
		if ( ptr->type & ( IS_NEW + IS_REVISION))
		     be_verbose( ptr, ptr->type);
		mystrncpy( ent_buf, ptr->ical_ent, MIDLEN - 40);
		if ( use_datebk3 &&
		     ptr->datebk_magic && ptr->datebk_magic[0] ) {
		        char *stuffed;
			int free_stuffed;
			cp = ent_buf + strlen( ent_buf) - 2;
			stuffed = str_bytestuff(ptr->datebk_magic, "[]", '\\');
			if (stuffed) {
			  free_stuffed = TRUE;
			} else {
			  free_stuffed = FALSE;
			  stuffed = ptr->datebk_magic;
			}
			sprintf(cp, "DateBk3Info [%s]\n]\n", stuffed);
			if (free_stuffed) free(stuffed);
			/* ptr->datebk_magic is unchanged in memory */
		}
		if ( !no_action)
			fputs( ent_buf, fp);
		ptr = ptr->next;
	}
	if ( no_action)
		return;

	mystrncpy( backname, cal_file, SMALLLEN);
	mystrncat( backname, ".bak", 2*SMALLLEN);
	if ( (stat( cal_file, &stat_buf) == -1) ||
	     				( rename( cal_file, backname) < 0 )) {
		if ( !quiet )
			fprintf(stderr, "Renaming %s failed: aborting\n", cal_file);
		cleanup( dlpEndCodeOther);
		exit( 1);
	}

	if ( (cfp = fopen( cal_file, "w")) == NULL ) {
		if ( !quiet )
			fprintf( stderr,
				 "Can't rewrite calendar file: %s aborting\n",
				 	cal_file);
		cleanup( dlpEndCodeOther);
		exit( 1);
	}
	ptr = lptr->first;
	fprintf( cfp, "Calendar [v2.0]\n");
	fprintf( cfp, extra_stuff); 
	while ( ptr != NULL ) {
		if ( ptr->category == 0) {
			fputs( ptr->ical_ent, cfp);
		}
		ptr = ptr->next;
	}
	fclose( cfp);
	chmod( cal_file, stat_buf.st_mode);
	default_mode = stat_buf.st_mode;

	for ( i = 1; i < MAX_PILOT_CATS; i++) {
		if ( (archv_cats[i][0] == '\0') || (cat_type[i] == CAT_ICAL))
			continue;
		mystrncpy( fname, archive_file, SMALLLEN);
		if ( (cp = strrchr( fname, '/')) == NULL)
			fname[0] = '\0';
		else
			*(++cp) = '\0';

		mystrncat( fname, archv_cats[i], SMALLLEN - 9);
		if ( cat_type[i] == CAT_PILOT)
			mystrncat( fname, ".Pilot", SMALLLEN - 4);

		mystrncpy( backname, fname, SMALLLEN - 4);
		mystrncat( backname, ".bak", SMALLLEN);

		if ( (stat( fname, &stat_buf) == -1) && (errno == ENOENT))
			stat_buf.st_mode = default_mode;
		else if ( rename( fname, backname) < 0 ) {
			if ( !quiet )
				fprintf(stderr, "Renaming %s failed: aborting\n",
					backname);
			cleanup( dlpEndCodeOther);
			exit( 1);
		}

		if ( (cfp = fopen( fname, "w")) == NULL ) {
			if ( !quiet )
				fprintf( stderr,
				       "Can't rewrite calendar file: %s aborting\n",
				 	fname);
			cleanup( dlpEndCodeOther);
			exit( 1);
		}

		ptr = lptr->first;
		fprintf( cfp, "Calendar [v2.0]\n");
		while ( ptr != NULL ) {
			if ( ptr->category == i) {
				fputs( ptr->ical_ent, cfp);
			}
			ptr = ptr->next;
		}
		fclose( cfp);
		chmod( fname, stat_buf.st_mode);

	}
}


	       
