/*
 * syncal_dates.c 
 * syncal Version 0.8.6
 *
 * Copyright (c) 1999-2000, John Franks
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "syncal.h"


static char
	*line_p,
	cur_token[MIDLEN],
	appointment_buf[MIDLEN],
	*set_tm( struct tm *, char *);

static void	
	appt_dates( struct Appointment *, int, int),
	sub_date( struct Appointment *, char *, int, int),
	contents2appt( char **, char **, struct A_entry * ),
	get_alarms( struct Appointment *),
	get_unknown( void),
	punt( struct Appointment *, char *, int, int);

static char *
set_tm( struct tm *datep, char *buf)
{

	char	*cp,
		*cp2,
		lbuf[MIDLEN];

	mystrncpy( lbuf, buf, MIDLEN);

	datep->tm_sec = datep->tm_min = datep->tm_hour = 0;
	datep->tm_isdst = 0;

	cp2 = skip( lbuf);
	if ( (cp = strchr( cp2, '/')) == NULL)
		format_error( "Bad date format (no '/')" );
	*cp++ = '\0';
	datep->tm_mday = atoi( cp2);
	cp2 = cp;
	if ( (cp = strchr( cp2, '/')) == NULL)
		format_error( "Bad date format (no '/')" );
	*cp++ = '\0';
	datep->tm_mon = atoi( cp2) - 1;
	cp2 = cp;
	while ( *cp && !isspace( *cp))
		cp++;
	if ( *cp )
		*cp++ = '\0';
	datep->tm_year = atoi( cp2) - 1900;
	return (char *) (buf + (cp - lbuf));
}

static void
appt_dates( struct Appointment *apt, int start, int len)
{
	struct tm
		adate, bdate, cdate;

	time_t	
		atime;
	char	*cp,
		buf[MIDLEN];


	get_lb( );
	get_token( );
	mystrncpy( buf, cur_token, MIDLEN);
	get_rb( );
	get_token( );

	apt->repeatFrequency = 0;
	apt->repeatDay = 0;
	apt->repeatWeekstart = 0;
	apt->exceptions = 0;
	apt->exception = NULL;
	apt->repeatForever = FALSE;
	apt->repeatEnd.tm_mday = 0;
	apt->repeatEnd.tm_mon = 0;
	apt->repeatEnd.tm_wday = 0;
	apt->repeatEnd.tm_year = 0;
	apt->repeatType = repeatNone;

	cp = buf;
	if ( strncasecmp( buf, "Single", 6) == 0) {
		cp += 6;
		cp = set_tm( &adate, cp);
		if ( (cp = strstr( cp, "End")) == NULL)
			format_error( "Missing 'End' for Dates entry" );

		cdate = bdate = adate;
		atime = gmt_mktime( &adate) + start*60;
		apt->begin = *(gmtime( &atime));

		atime += len*60;
		apt->end = *(gmtime( &atime));
	}
	else if ( strncasecmp( cp, "Days", 4) == 0) {
		int	freq;

		cp += 4;
		apt->repeatType = repeatDaily;
		apt->repeatForever = TRUE;
		cp = skip( cp);
		while ( !isspace( *cp))
			cp++;  /* skip unused date */
		cp = skip( cp);
		freq = atoi( cp);
		apt->repeatFrequency = freq;
		while ( !isspace( *cp))
			cp++;

		sub_date( apt, cp, start, len);
		/* change repeat every 7 days to repeat weekly */
		if ( freq % 7 == 0 ) {
			int j;

			apt->repeatFrequency = freq/7;
			apt->repeatType = repeatWeekly;
			for (j = 0; j < 7; j++) {
				apt->repeatDays[j] = FALSE;
			}
			j = apt->begin.tm_wday;
			apt->repeatDays[j] = TRUE;
		}
	}
	else if ( strncasecmp( cp, "Months", 6) == 0) {
		int	freq;

		cp += 6;
		apt->repeatType = repeatMonthlyByDate;
		apt->repeatForever = TRUE;
		cp = skip( cp);
		while ( !isspace( *cp))
			cp++;  /* skip unused date */
		cp = skip( cp);
		freq = atoi( cp);
		if ( freq % 12 == 0 ) {
			freq = freq/12;
			apt->repeatType = repeatYearly;
		}
		apt->repeatFrequency = freq;
		while ( !isspace( *cp))
			cp++;
		sub_date( apt, cp, start, len);
	}
	else if ( strncasecmp( cp, "WeekDays", 8) == 0) {
		int	i;

		cp += 8;
		for ( i = 0; i < 7; i++ ) {
			apt->repeatDays[i] = FALSE;
		}
		apt->repeatType = repeatWeekly;
		apt->repeatFrequency = 1;
		apt->repeatForever = TRUE;
		cp = skip( cp);
		while ( isdigit( *cp)) {
			i = (int) (*cp - '0');
			if ( (i < 1) || (i > 7))
				format_error( "Day of week out of range" );
			apt->repeatDays[i - 1] = TRUE;
			cp++;
			cp = skip( cp);
		}
		cp = strchr( cp, '\n');  /* skip to end of line */
		sub_date( apt, cp, start, len);
	}
	else if ( strncasecmp( cp, "ComplexMonths", 13) == 0) {
		char	*cp2;
		int	freq,
			willpunt,
			myweek,
			dowk;

		willpunt = FALSE;
		apt->repeatType = repeatMonthlyByDay;
		apt->repeatForever = TRUE;

		cp += 13;
		cp = skip( cp);
		cp2 = cp;
		while ( isdigit( *cp2))
			cp2++;
		*cp2++ = '\0';
		freq = atoi( cp);
		apt->repeatFrequency = freq;
		cp = cp2;


		cp = skip( cp);
		cp2 = cp;
		while ( isdigit( *cp2))
			cp2++;
		*cp2++ = '\0';
		myweek = atoi( cp) - 1;
		if ( (myweek > 4) || (myweek < 0))
			format_error( "Week of month out of range" );
		cp = cp2;

		cp2 = strstr( cp, "Backward");
		if ( cp2 != NULL) {
			if ( myweek == 0 )
				myweek = 4; /* first week from end is last week */
			else
				willpunt = TRUE;
		}
		else if ( (cp2 = strstr( cp, "Forward")) == NULL)
			willpunt = TRUE;

		cp2 = strstr( cp, "ByWeek");
		if ( willpunt || (cp2 == NULL)) {
			cp = strchr( cp, '\n');
			punt( apt, cp, start, len);
			return;
		}

		cp = cp2;
		cp += 6;
		cp = skip( cp);
		dowk = atoi( cp) - 1;
		if ( (dowk > 6) || (dowk < 0))
			format_error( "Day of week out of range");

		apt->repeatDay = (enum DayOfMonthType) 7 * myweek + dowk;

		cp = strchr( cp, '\n');  /* skip to end of line */
		sub_date( apt, cp, start, len);
	}
}


static void
punt( struct Appointment *ptr, char *cp, int start, int len)
{
	if ( strstr( cp, "End") == NULL)
		format_error( "Missing 'End' for Dates entry" );
	sub_date( ptr, cp, start, len);
	ptr->repeatFrequency = 0;
	ptr->repeatWeekstart = 0;
	ptr->exceptions = 0;
	ptr->exception = NULL;
	ptr->repeatForever = FALSE;
	ptr->repeatEnd.tm_mday = 0;
	ptr->repeatEnd.tm_mon = 0;
	ptr->repeatEnd.tm_wday = 0;
	ptr->repeatType = repeatNone;

	if ( !quiet ) {
		fprintf(stderr, "I don't understand the repetition for:\n\t%s\n",
		 ptr->description);
		fprintf( stderr, "Entering with no repetition.\n");
	}
}
		 

char *
skip( char *ptr)
{
	while ( *ptr && isspace( *ptr))
		ptr++;
	return ptr;
}


static void
sub_date( struct Appointment *apt, char *buf, int start, int len)
{
	struct tm adate;
	time_t	atime;

	char	*cp;

	if ( (cp = strstr( buf, "End")) == NULL)
		format_error( "Missing 'End' for Dates entry" );
	else
		*cp = '\0';
	     
	buf = skip( buf);
	while ( *buf ) {
		if ( strncasecmp( buf, "Start", 5) == 0) {
			buf += 5;
			buf = set_tm( &adate, buf);
			atime = gmt_mktime( &adate) + start*60;
			apt->begin = *(gmtime( &atime));
			atime += len*60;
			apt->end = *(gmtime( &atime));
		}
		else if ( strncasecmp( buf, "Finish", 6) == 0) {
			buf += 6;
			buf = set_tm( &adate, buf);
			atime = gmt_mktime( &adate) + start*60;
			apt->repeatForever = FALSE;
			apt->repeatEnd = *(gmtime( &atime));
		}
		else if ( strncasecmp( buf, "Deleted", 7) == 0) {
			struct tm *ptr;
			int	j;

			buf += 7;
			j = apt->exceptions;
			ptr = apt->exception;
			ptr = (struct tm *) realloc( ptr, 
						     sizeof( struct tm) * (j + 2));
			if ( ptr == NULL) {
				if ( !quiet )
					fprintf( stderr, "Out of memory\n");
				cleanup( dlpEndCodeOutOfMemory);
				exit(1);
			}
			buf = set_tm( &adate, buf);
			atime = gmt_mktime( &adate) + start*60;
			ptr[j] = *(gmtime( &atime));
			apt->exception = ptr;
			apt->exceptions++;
		}
		else {
			format_error( "I don't understand this Dates entry" );
		}
		buf = skip( buf);
	}
}

/* 
 * get_token( ) gets the next token and places it in cur_token
 * Tokens are separated by '[' and ']'.  Whitespace is deleted at
 * the beginning and end of a token.
 */

void
get_token( void)
{

	char	*cp;

	while ( *line_p && isspace( *line_p))
		line_p++;

	if ( !*line_p ) { 	/* end of input */
		cur_token[0] = '\0';
		return;
	}

	if ( (*line_p == '[') || (*line_p == ']') ) {
		cur_token[0] = *line_p++;
		cur_token[1] = '\0';
		return;
	}

	cp = cur_token;

	while ( *line_p) {
		switch( *line_p) {
		case '[':
		case ']':
			*cp-- = '\0';
			while ( isspace( *cp)) {
				*cp-- = '\0';
			}
			return;
		case '\\':
			if ( (*(line_p +1) == '[') || (*(line_p +1) == ']'))
				line_p++;
			*cp++ = *line_p++;
			break;
		default:
			*cp++ = *line_p++;
		}
	}
	*cp-- = '\0';
	while ( isspace( *cp)) {
		*cp-- = '\0';
	}

}

/* 
 * get_lb( ) and get_rb( ) eat the next '[' and ']' respectively and
 * issue an error if they aren't there.
 */

void
get_lb( void)
{
	get_token( );
	if ( strcmp( cur_token, "[") != 0)  {
		format_error( "Missing '['" );
	}
}


void
get_rb( void)
{
	get_token( );
	if ( strcmp( cur_token, "]") != 0)  {
		format_error( "Missing ']'" );
	}
}


/*
 * void fill_appt(  struct A_entry *ep, struct A_list *lptr )
 * Assumes that ep->ical_ent, ep->category, ep->db3magic and ep->type are 
 * there and translates their information to fill ep->appt appropriately.
 */

void
fill_appt( struct A_entry *ep, struct A_list *lptr )
{
	struct Appointment * ap;
	char	rec_id_str[SMALLLEN];

	static char	*desc_p,
			*note_p;

	int	length,
		start;

	ap = &(ep->appt);
	start = 0;
	length = 0;
	mystrncpy( appointment_buf, ep->ical_ent, MIDLEN);
	line_p = appointment_buf;

	rec_id_str[0] = '\0';
	cur_token[0] = '\0';
	get_token( );
	get_lb( );
	get_token( );

	desc_p = note_p = NULL;
	ap->event = ( (ep->type & IS_EVENT) ? TRUE : FALSE);
	ap->description = NULL;
	ap->note = NULL;
	ap->alarm = FALSE;
	ap->advance = 0;
	ap->advanceUnits = 0;

	while ( cur_token[0] ) {
		if ( strcmp( cur_token, "Text") == 0 ) {
		  	fprintf( stderr, ERRMSG1, "< 2.0");
		  	fprintf( stderr, ERRMSG2);
			exit( 1);
		}
		if ( strcmp( cur_token, "Contents") == 0 )
			contents2appt( &desc_p, &note_p, ep);
		else if ( strcasecmp( cur_token, "Start") == 0 )
			start = get_int( );
		else if ( strcasecmp( cur_token, "Length") == 0 ) {
			length = get_int( );
			if ( ap->event)
				length = 0;
		}
		else if ( strcasecmp( cur_token, "Alarms") == 0 )
			get_alarms( ap);
		else if ( strcasecmp( cur_token, "Dates") == 0 )
			appt_dates( ap, start, length );
		else if ( strcmp( cur_token, "]") == 0 )
			break;
		else
			get_unknown( );
	}

	if ( (!ap->event) && (!ap->alarm) && lptr &&
				(lptr->default_alarm != ALARM_NONE)) {
		int val;

		val = lptr->default_alarm;

		if ( (val % 60) == 0 ) {
			val = val/60;
			ap->advanceUnits = ALARM_HOURS;
		}
		else
			ap->advanceUnits = ALARM_MINUTES;

		ap->advance = val;
		ap->alarm = TRUE;
	}

	ap->description = desc_p;
	if ( note_p )
		ap->note = note_p;

	if ( ep->datebk_magic && ep->datebk_magic[0]
				&& (ep->datebk_magic[6] == 's')) {
		time_t	atime;
		/* Handle datebk3 appts which span midnight */
		ap->repeatType = repeatDaily;
		ap->repeatForever = FALSE;
		ap->repeatFrequency = 1;
		atime = gmt_mktime( &(ap->begin));
		atime += 24*60*60;
		ap->repeatEnd = *(gmtime( &atime));
	}
}

static void
get_alarms( struct Appointment *apt)
{
	int	val;
	char	*cp;

	get_lb( );
	get_token( );
	if ( strcmp( cur_token, "]") == 0)  {  /* It's Alarms [] */
		get_token( );
		return;
	}

	cp = cur_token;
	while ( *cp )
		cp++;
	cp--;
	while ( isspace( *cp) && ( cp > cur_token))
		cp--;
	while ( !isspace( *cp) && ( cp > cur_token))
		cp--;

	val = atoi( cp);
	if ( (val % 60) == 0 ) {
		val = val/60;
		apt->advanceUnits = ALARM_HOURS;
	}
	else
		apt->advanceUnits = ALARM_MINUTES;

	apt->advance = val;
	apt->alarm = TRUE;

	get_rb( );
	get_token( );
}

int
get_int( void)
{
	int val;

	get_lb( );
	get_token( );

	if (!isdigit( cur_token[0]))
		val = (-1);
	else
		val = atoi( cur_token);

	get_rb( );
	get_token( );
	return (val);
}


static void 
contents2appt(char **descptr, char **nptr, struct A_entry *ep)
{
	char	notebuf[MIDLEN],
		*cp;

	notebuf[0] = '\0';
	if ( use_datebk3) {
		if ( ep->datebk_magic && ep->datebk_magic[0] != '\0')
			mystrncpy( notebuf, ep->datebk_magic, MIDLEN);

		if (( ep->category != 0) || (ep->type & (IS_TODO + IS_DONE))) {
		        if ( !ep->datebk_magic) {
			        /* we add a DateBk3 size tag */
			        ep->datebk_magic = (char*)malloc(10);
				ep->datebk_magic[0] = '\0';
			}
			if ( ep->datebk_magic[0] == '\0') {
				mystrncpy( ep->datebk_magic, "##@@@@@@@\n", 11);
				mystrncpy( notebuf,      "##@@@@@@@\n", 11);
                        }
			
			cp = notebuf;
			if ( ep->type & IS_TODO)
				ep->datebk_magic[2] = cp[2] = 'f';
			if ( ep->type & IS_DONE)
				ep->datebk_magic[2] = cp[2] = 'c';
			if ( (ep->category > 0) && (ep->category < MAX_PILOT_CATS))
				ep->datebk_magic[4] = cp[4] = '@' + ep->category;
			if ( strcmp( cp, "##@@@@@@@\n") == 0 ) {
			        free(ep->datebk_magic);
				ep->datebk_magic = NULL;
				cp[0] = '\0';
			}
		}
	}
	get_lb();
	get_token();
	if ( strcmp( cur_token, "]") == 0 ) { 	/* Empty contents */
		*descptr = strdup( "");
		get_token();
		return;
	}
	if ((cp = strstr(cur_token, "\n\n")) != NULL) {
		*cp++ = '\0';
		*cp++ = '\0';
		mystrncat( notebuf, cp, PI_NOTE_LEN);
	}
	if ( notebuf[0] ) {
		*nptr = strdup( notebuf);
		if ( *nptr == NULL) {
			if ( !quiet )
				fprintf( stderr, "Out of memory\n");
			cleanup( dlpEndCodeOutOfMemory);
			exit(1);
		}
	}
	*descptr = strdup( cur_token);
	if ( *descptr == NULL) {
		if ( !quiet )
			fprintf( stderr, "Out of memory\n");
		cleanup( dlpEndCodeOutOfMemory);
		exit(1);
	}
	if ( strlen( *descptr) >= PI_DESC_LEN )
		(*descptr)[PI_DESC_LEN] = '\0';
	get_rb();
	get_token( );
}

static void
get_unknown( void )
{
	int	brack_count;

	get_lb( );
	brack_count = 1;

	while ( *line_p && brack_count > 0) {
		get_token( );
		if ( strcmp( cur_token, "[") == 0 )
			brack_count++;
		if ( strcmp( cur_token, "]") == 0 )
			brack_count--;
	}
	if ( brack_count != 0 )
		format_error( "Missing ']' at end of record");
	get_token( );
}

void
format_error( char * msg)
{
	if ( !quiet )
		fprintf( stderr, "Format error\n");
	if ( *msg && !quiet )
		fprintf( stderr, "\t%s\n", msg);
	if ( !quiet )
		fprintf( stderr, "\t%s\n", appointment_buf );

	cleanup( dlpEndCodeOther);
	exit( 1);
}

