/*
 * syncal_list.c
 * syncal Version 0.8.6
 *
 * Copyright (c) 1999-2000, John Franks
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "syncal.h"

static int
	ecomp( struct A_entry **, struct A_entry ** );

static void
	free_Aentry( struct A_entry *);


/*
 * ecomp( struct A_entry **p1, struct A_entry **p2)
 * comparison function for sorting array of points to A_entries.
 * It returns the sign of the  numeric differenct in the pi_rec_id's 
 * if they are different and zero if they are the same.
 */

static int
ecomp( struct A_entry **p1, struct A_entry **p2)
{
	long	val;

	val = (long) ((*p1)->pi_rec_id - (*p2)->pi_rec_id );

	if ( val > 0)
		return (1);
	else if ( val < 0 )
		return (-1);
	return (0);
		/*	return ((*p1)->type - (*p2)->type ); */
}



/*
 * void sort( struct A_list *lptr)
 * Takes a struct A_list * lptr which points to  a linked list of A_entries.  
 * List is sorted in increasing order of  pilot record id of entries.
 */

void
sort_list( struct A_list *lptr)
{
	struct A_entry	
		**sp,
		*ptr;

	int	i;
	
	sp = (void *) malloc( (lptr->num + 1) * sizeof( struct A_entry));
	if ( sp  == NULL) {
		if ( !quiet )
			fprintf( stderr, "Out of memory\n");
		cleanup( dlpEndCodeOutOfMemory);
		exit( 1);
	}

	ptr = lptr->first;

	i = 0;
	while ( ptr != NULL) {
		sp[i++] = ptr;
		ptr = ptr->next;
	}
	sp[i] = NULL;
	lptr->num = i;

	qsort( sp, lptr->num, sizeof ( struct A_entry *), (int (*)( )) ecomp );

	/* now redo the links so the linked list corresponds to array */
	lptr->first = sp[0];
	for ( i = 0; i < lptr->num; i++ )
		sp[i]->next = sp[i+1];

	free( sp);
}


static void
free_Aentry( struct A_entry *ep)
{
	if (ep) {
		free_Appointment( &(ep->appt));
		free( ep->ical_ent);
		if ( ep->owner)
			free( ep->owner);
		if ( ep->datebk_magic)
			free( ep->datebk_magic);
		free( ep);
	}
}


void
free_Alist( struct A_list *lptr)
{
	struct A_entry
		*lep,
		*next;

	lep = lptr->first;

	while ( lep != NULL) {
		next = lep->next;
		free_Aentry( lep);
		lep = next;
	}
}


/*
 * mystrncpy( s1, s2, n) is a strncpy() which guarantees a null
 * terminated string in s1.  At most (n-1) chars are copied.
 * Returns -1 if truncation occurred and (n-1) minus number of
 * bytes copied otherwise.
 */

int
mystrncpy( s1, s2, n)
char	*s1,
	*s2;
int	n;
{
	register char	*cp1,
			*cp2;
	cp1 = s1;
	cp2 = s2;
	n--;

	while ( *cp2 && (n > 0)) {
		n--;
		*cp1++ = *cp2++;
	}
	*cp1 = '\0';
	if ( *cp2 ) {
		if ( !quiet )
			fprintf( stderr, "Item too long: truncated\n%s\n", s1);
	}
	return (n);
}

/*
 * mystrncat( s1, s2, n) is a strncat() which guarantees a null
 * terminated string in s1.  At most (n-1) chars TOTAL are in the
 * concatenated string.  If the original s1 had more than that
 * it is truncated.
 * Returns -1 if truncation occurred and (n-1) minus number of
 * bytes in new s1 otherwise.
 */

int
mystrncat( s1, s2, n)
char	*s1,
	*s2;
int	n;
{
	register char	*cp1,
			*cp2;
	cp1 = s1;
	cp2 = s2;
	n--;

	while ( *cp1 && (n > 0)) {
		n--;
		cp1++;
	}
	if ( n == 0 ) {
		if ( *cp1 ) {
			*cp1 = '\0';
			if ( !quiet )
				fprintf( stderr,
					 "Item too long: truncated\n%s\n", s1);
			return (-1);
		}
		return (0);
	}

	while ( *cp2 && (n > 0)) {
		n--;
		*cp1++ = *cp2++;
	}
	*cp1 = '\0';
	if ( *cp2 ) {
		if ( !quiet )
			fprintf( stderr, "Item too long: truncated\n%s\n", s1);
		return (-1);
	}
	return (n);
}


